<?php
/**
 * @see Tool_InfoController
 */
require_once GT_MODULES_PATH . DIR_SEP . 'tool' . DIR_SEP . 'controllers' . DIR_SEP . 'JobsController.php';

class Mobile_JobsController extends Tool_JobsController
{

    public function getAction ()
    {
        $result['replyCode'] = 200;

        $config = Zend_Registry::get('config');
        $request = $this->getRequest();
        
        $completed = ($request->getParam('completed') == 'true') ? true : false;
        $offset = intval($request->getParam('offset'));

        $jobsTable = new Giscuit_Db_Table_Jobs();

        
        //Set database params
        $jobParams = array(
            'withActions' => true,
            'whereCurrentUser' => true
        );
        if ($completed) {
            $jobParams['whereStatus'] = "('completed')";
            $jobParams['limit'] = 20;
            $jobParams['offset'] = $offset;
            $jobParams['order'] = 'date DESC';
        } else {
            $jobParams['whereStatus'] = "('pending', 'in_progress')";
        }
        
        $jobs = $jobsTable->getItems($jobParams);
        
        //Get workflow actions
        if (! empty($jobs)) {
            $result['actions'] = $this->getWorkflowActions($jobs);
        }

        $result['records'] = $jobs;

        $this->_helper->json($result);
    }
    
    public function updateactionAction()
    {
        $result['replyCode'] = 200;
        
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        $request = $this->getRequest();
        $newActions = $request->getParam('actions');
        $jobActionsTable = new Giscuit_Db_Table_JobsActions();
        $jobsTable = new Giscuit_Db_Table_Jobs();
        
        //Parse actions
        $actions = array();
        foreach ($newActions as $action) {
            //Parse geometry
            $parsedGeometry = explode(' ', substr(substr($action['geometry'], 6), 0, -1));
            
            $parsedAction = array(
                'jobId' => intval($action['id']),
                'action' => intval($action['action']),
                'comment' => htmlspecialchars($action['comment']),
                'coordX' => floatval($parsedGeometry[0]),
                'coordY' => floatval($parsedGeometry[1])
            );
            $actions[] = $parsedAction;
            
            $reservedAction = intval($action['reserved']);
            if ($reservedAction == 1) {
                $jobsTable->updateStatus($parsedAction['jobId'], 'in_progress');
            } else if ($reservedAction == 2) {
                $jobsTable->updateStatus($parsedAction['jobId'], 'declined');
            } else if ($reservedAction == 3) {
                $jobsTable->updateStatus($parsedAction['jobId'], 'completed');
            }
            
            //Email notify
            if ($config->smtp->host && $config->smtp->port) {
                $jobRecord = $jobsTable->getItems(array('whereId' => $parsedAction['jobId']));
                $jobRecord = $jobRecord[0];
                $usersTable = new Giscuit_Db_Table_Users();
                $userRecords = $usersTable->getItems(array(
                    'whereIdsIn' => $jobRecord['from_user_id'] . ', ' . $jobRecord['to_user_id'],
                    'withIdentity' => true
                ));
                
                foreach ($userRecords as $record) {
                    if ($record->id == $jobRecord['from_user_id']) {
                        $dispatcher = $record;
                    }
                    if ($record->id == $jobRecord['to_user_id']) {
                        $worker = $record;
                    }
                }
                
                if ($jobRecord['notify'] == 2 && (! $reservedAction || $reservedAction == 4) &&
                    ! empty($dispatcher->email) && $dispatcher->email_active == 't') {
                    
                    if ($parsedAction['action'] == -1) {
                        $actionName = $translate->_('comment');
                    } else {
                        $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap;
                        $action = $actionsMapTable->getItems(array(
                            'whereId' => $parsedAction['action'],
                            'withMultilangFields' => true
                        ))->current();
                        
                        $actionName = $action['name_en'];
                    }
                    
                    $this->sendNewActionMail(array(
                        'title' => $jobRecord['title'],
                        'actionName' => $actionName,
                        'comment' => $parsedAction['comment'],
                        'worker' => $worker->identity,
                        'email' => $dispatcher->email
                    ));
                    
                } else if ($jobRecord['notify'] == 3 && ($reservedAction && $reservedAction != 4) &&
                    ! empty($dispatcher->email) && $dispatcher->email_active == 't') {
                    if ($reservedAction == 1) {
                        $subject = $translate->_('taskAccepted') . ': ' . $jobRecord['title'];
                    } else if ($reservedAction == 2) {
                        $subject = $translate->_('taskDeclined') . ': ' . $jobRecord['title'];
                    } else if ($reservedAction == 3) {
                        $subject = $translate->_('taskCompleted') . ': ' . $jobRecord['title'];
                    }
                    
                    $this->sendStatusMail(array_merge($jobRecord, array(
                        'worker' => $worker->identity,
                        'email' => $dispatcher->email,
                        'subject' => $subject
                    )));
                }
            }
            
        }
        
        foreach ($actions as $action) {
            $jobActionsTable->addItem($action);
        }
        
        $this->_helper->json($result);
    }
    
    public function sendStatusMail($data)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $mailView = new Zend_View();
        $mailView->title = $data['title'];
        $mailView->worker = $data['worker'];
        $mailView->text = $data['subject'];
        
        $mailView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'tool' . DIR_SEP . 
            'views' . DIR_SEP . 'scripts' . DIR_SEP . 'jobs');
        
        $mailBody = $mailView->render('jobstatus_mail' . GT_SCRIPT_EXT);

        Giscuit_Mail::batch(array(
            array(
                'to' => $data['email'] ,
                'subject' => $data['subject'] , 
                'text' => $mailBody
            )
        ));
    }
    
    public function sendNewActionMail($data)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $mailView = new Zend_View();
        $mailView->title = $data['title'];
        $mailView->actionName = $data['actionName'];
        $mailView->worker = $data['worker'];
        
        if (! empty($data['comment'])) {
            $mailView->comment = $data['comment'];
        }
        
        $mailView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'tool' . DIR_SEP . 
            'views' . DIR_SEP . 'scripts' . DIR_SEP . 'jobs');
        
        $mailBody = $mailView->render('newaction_mail' . GT_SCRIPT_EXT);

        Giscuit_Mail::batch(array(
            array(
                'to' => $data['email'] ,
                'subject' => $data['actionName'] . ': ' . $data['title'], 
                'text' => $mailBody
            )
        ));
    }

}