<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: RastercutController.php 3340 2013-03-15 12:24:45Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_RastercutController extends Zend_Controller_Action
{

    protected $_tileWidth = 5000;

    protected $_tileHeight = 5000;

    protected $_maxImagePerimeter = 10000;
    
    public function indexAction ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');
        
        $request = $this->getRequest();
        
        $result['replyCode'] = 200;
        $result['valid'] = false;
        
        //Input
        $form = $this->getForm();
        $form->populate($request->getParams());
        
        if ($form->isValid($request->getParams())) {
            
            $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
            
            $geometrySql = "ST_GeometryFromText('" . $form->getValue('geometry') . "', {$config->map->srid})";
            $extentSql = "SELECT ST_Extent($geometrySql) AS the_geom";
            
            $extent = Giscuit_Map_Map::bboxToExtent($db->fetchRow($extentSql)->the_geom);
            $info = Giscuit_Map_Map::getInfoFromExtent($extent, $form->getValue('scale'), $form->getValue('resolution'));
            
            if (($info['size']['width'] + $info['size']['height']) > $this->_maxImagePerimeter && $form->getValue('tiled') == 0) {
                $form->getElement('geometry')->addError($translate->_('imageTooLarge'));
            } else {
                $result['file'] = $file = md5(uniqid(rand(), true));
                
                //Watermark
                $date = new Zend_Date();
                $auth = Zend_Auth::getInstance();
                if ($isAllowed->direct('rastercut_watermark')) {
                    $watermarkText = $form->getValue('comment');
                } else {
                    $watermarkText = $auth->getIdentity()->fname . ' ' . $auth->getIdentity()->lname . ' ' . $date->get(GT_DATE_STYLE_PHP);
                }
                
                $i = 0;
                
                if ($form->getValue('tiled') == 1) {
                    $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
                    
                    if ($isAllowed->direct('rastercut_tiled')) {
                        $tiles = $this->getTiles($info, $geometrySql);
                        
                        foreach ($tiles as $extent) {
                            Giscuit_Map_Map::getImage($file . '_' . $i, array(
                                'resolution' => $form->getValue('resolution') , 
                                'extent' => $extent , 
                                'scale' => $form->getValue('scale') , 
                                'burnPolygon' => $geometrySql , 
                                'watermark' => $watermarkText
                            ));
                            
                            $i ++;
                        }
                    
                    }
                
                } else {
                    
                    Giscuit_Map_Map::getImage($file, array(
                        'resolution' => $form->getValue('resolution') , 
                        'extent' => $extent , 
                        'scale' => $form->getValue('scale') , 
                        'burnPolygon' => $geometrySql , 
                        'watermark' => $watermarkText
                    ));
                
                }
                
                $this->archive($file, $i);
                
                $rastercutTable = new Giscuit_Db_Table_Layer_Rastercut(array(
                    'name' => 'rastercut'
                ));
                $rastercutTable->addItem($form);
                
                $result['valid'] = true;
            }
        
        }
        
        if ($result['valid'] == false) {
            $result['form'] = $form->render();
        
        }
        
        $this->_helper->json($result);
    }

    public function downloadAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $file = $this->getRequest()->getParam('file');
        
        if (empty($file) || ! preg_match('/^[a-f0-9]{32}$/', $file)) {
            return;
        }
        
        $fullPath = GT_TMP_PATH . DIR_SEP . $file . '.zip';
        if ($fd = fopen($fullPath, "r")) {
            header("Content-type: application/octet-stream");
            header('Content-Disposition: filename="giscuit_map.zip"');
            header("Content-length: " . filesize($fullPath));
            header("Cache-control: private");
            while (! feof($fd)) {
                $buffer = fread($fd, 2048);
                echo $buffer;
            }
        }
        fclose($fd);
    }
    
    public function archive ($file, $tiled = 0)
    {
        $zip = new ZipArchive();
        
        $fullPath = GT_TMP_PATH . DIR_SEP . $file;
        
        $res = $zip->open($fullPath . '.zip', ZipArchive::CREATE);
        if (! $res) {
            throw new Giscuit_Exception("Cannot open archive for writing");

        }
        
        //Single tile
        $delList = array();
        
        //Single tile
        if ($tiled == 0) {
            $zip->addFile($fullPath . '.jpg', 'map' . '.jpg');
            $delList[] = $fullPath . '.jpg';

            $zip->addFile($fullPath . '.jgw', 'map' . '.jgw');
            $delList[] = $fullPath . '.jgw';

        } else {
            for ($i = 0; $i < $tiled; $i ++) {
                $zip->addFile($fullPath . '_' . $i . '.jpg', $i . '.jpg');
                $delList[] = $fullPath . '_' . $i . '.jpg';

                $zip->addFile($fullPath . '_' . $i . '.jgw', $i . '.jgw');
                $delList[] = $fullPath . '_' . $i . '.jgw';
            }
        }
        
        $zip->close();
        
        foreach ($delList as $value) {
            unlink($value);
        }
    
    }

    public function getTiles ($info, $geometry)
    {
        $tiles = array();
        
        $wc = ceil($info['size']['width'] / $this->_tileWidth);
        $wh = ceil($info['size']['height'] / $this->_tileHeight);
        
        for ($a = 0; $a < $wh; $a ++) {
            for ($b = 0; $b < $wc; $b ++) {
                
                $x1 = $y1 = $x2 = $y2 = NULL;
                
                $x1 = $info['extent']['min']['x'] + $b * $this->_tileWidth * $info['pixel']['width'];
                $y1 = $info['extent']['min']['y'] + $a * $this->_tileWidth * $info['pixel']['height'];
                
                $x2 = $info['extent']['min']['x'] + ($b + 1) * $this->_tileWidth * $info['pixel']['width'];
                $y2 = $info['extent']['min']['y'] + ($a + 1) * $this->_tileWidth * $info['pixel']['height'];
                
                $extent = array(
                    'min' => array(
                        'x' => $x1 , 
                        'y' => $y1
                    ) , 
                    'max' => array(
                        'x' => $x2 , 
                        'y' => $y2
                    )
                );
                
                if ($this->_intersects($extent, $geometry)) {
                    $tiles[] = $extent;
                } else {

                }
            }
        }
        
        return $tiles;
    }

    protected function _intersects ($extent, $geometry)
    {
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
        
        $bbox = "ST_SetSRID('BOX3D({$extent['min']['x']} {$extent['min']['y']}, {$extent['max']['x']} {$extent['max']['y']})'::box3d, {$config->map->srid})";
        $sql = "SELECT st_intersects({$bbox}, {$geometry}) as intersects";
        
        return (bool) $db->fetchRow($sql)->intersects;
    
    }
    
    public function getformAction ()
    {
        $this->view->form = $this->getForm();
    }

    public function getForm ()
    {
        $form = new Giscuit_Form_Rastercut();
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtRastercutForm' , 
            'onsubmit' => 'return false;'
        ));
        
        return $form;
    }

    public function manageAction ()
    {
        $form = $this->getManageForm();
        
        $this->view->form = $form;
    }

    public function managedatatableAction ()
    {
        $result['replyCode'] = 200;
        
        $settings = Zend_Registry::get('settings');
        
        $request = $this->getRequest();
        
        $form = $this->getManageForm();
        $form->populate($request->getParams());
        
        if (! $form->isValid($request->getParams())) {
            $result['records'] = array();
        } else {
            $settings->rastercutDateBegin = $request->getParam('dateBegin');
            $settings->rastercutDateEnd = $request->getParam('dateEnd');
            
            $layersTable = new Giscuit_Db_Table_Layers();
            $rastercutLayer = array_values($layersTable->getItems(array(
                'whereClass' => 'Rastercut' , 
                'withFields' => true
            )));
            
            $rastercut = new Giscuit_Db_Table_Layer_Rastercut(array(
                'name' => $rastercutLayer[0]['table']
            ));
            $rastercut->setSettings($rastercutLayer[0]);
            
            $rows = $rastercut->getByDate();
            
            $result['records'] = $rows->toArray();
            
        }
        
        $this->_helper->json($result);
    
    }

    public function getManageForm ()
    {
        $form = new Giscuit_Form_RastercutManage();
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtRastercutManageForm' , 
            'onsubmit' => 'return false;'
        ));
        
        return $form;
    }

}