<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: IsAllowed.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Helper
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see  Zend_Controller_Action_Helper_Abstract
 */
require_once 'Zend/Controller/Action/Helper/Abstract.php';

/**
 * Check if a given resource and privilege is allowed for current user
 * 
 * @category   Giscuit
 * @package    Giscuit_Helper
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Action_Helper_IsAllowed extends Zend_Controller_Action_Helper_Abstract
{

    /*
    public $pluginLoader;

    public function __construct ()
    {
        $this->pluginLoader = new Zend_Loader_PluginLoader();
    }
    */
    
    /**
     * Check if a given resource and privilege is allowed for current user
     * 
     * @param string $resource Resource 
     * @param string $privilege Privilege
     * @return bool
     */
    public function isAllowed ($resource, $privilege = NULL)
    {
        
        if (empty($resource)) {
            false;
        }
        
        $auth = Zend_Auth::getInstance();
        if (! $auth->hasIdentity()) {
            false;
        }
        
        if (! Zend_Registry::isRegistered('acl')) {
            return false;
        }
        
        if (empty($auth->getIdentity()->group)) {
            $group = 'guest';
        } else {
            $group = $auth->getIdentity()->group;
        }
        
        $acl = Zend_Registry::get('acl');
        
        try {
            return $acl->isAllowed($group, $resource, $privilege);
        } catch (Zend_Acl_Exception $e) {
            return false;
        }
    
    }
    
    /**
     * Strategy pattern: call helper as helper broker method
     * 
     * Check if a given resource and privilege is allowed for current user
     * 
     * @param string $resource Resource 
     * @param string $privilege Privilege
     * @return bool
     */
    public function direct ($resource, $privilege = NULL)
    {
        return $this->isAllowed($resource, $privilege);
    }

}