<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Articles.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Action
 */
require_once 'Zend/Controller/Action.php';

/**
 * Admin articles controller  
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Admin_Articles extends Zend_Controller_Action
{

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        parent::init();
        
        $this->_table = new $this->_tableClassName();
    }

    /**
     * Dispatch index action
     * 
     * @return void
     */
    public function indexAction ()
    {
        $this->view->recordsPerPage = $this->_recordsPerPage;
        $this->view->controllerName = $this->_controllerName;
        $this->view->totalRecords = $this->_table->count();
    }

    /**
     * Dispatch get action
     * 
     * @return void
     */
    public function getAction ()
    {
        $result['replyCode'] = 200;
        
        $rows = $this->_table->getItems(array(
            'withTitle' => true
        ));
        
        $this->_fetchCallback($rows);
        
        $result['records'] = $rows->toArray();
        
        $this->_helper->json($result);
    }

    /**
     * Dispatch add action
     * 
     * @return void
     */
    public function addAction ()
    {
        
        //Input
        $request = $this->getRequest();
        $browse = $request->getUserParam('browse');
        $form = $this->getAddForm();
        
        //Hide createdDate
        $form->removeElement('created_date');
        
        if (empty($browse)) {

        } else {
            $form->populate($_POST);
            
            //Invalid input data
            if (! $form->isValid($_POST)) {

            } else {
                
                //Disable view and layout
                $layout = Zend_Layout::getMvcInstance();
                if ($layout instanceof Zend_Layout) {
                    $layout->disableLayout();
                }
                $this->_helper->viewRenderer->setNoRender();
                
                $this->_table->addItem($form);
                
                //Redirect to current controller
                $this->_helper->redirector->gotoRoute(array(
                    'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
                    'module' => 'admin' ,
                    'controller' => $this->_controllerName , 
                    'module' => 'admin'
                ), null, true);
            }
        }
        $this->view->form = $form;
        $this->view->languages = Zend_Registry::get('config')->language->toArray();
    }

    /**
     * Dispatch edit action
     * 
     * @return void
     */
    public function editAction ()
    {
        $this->view->controllerName = $this->_controllerName;
        
        //Input
        $request = $this->getRequest();
        $browse = $request->getUserParam('browse');
        $id = intval($request->getUserParam('id'));
        $form = $this->getEditForm($id);
        if (empty($browse)) {
            
            $row = $this->_table->getItems(array(
                'withMultilangFields' => true , 
                'whereId' => $id
            ));
            
            if(count($row) == 0) {
                throw new Exception("Invalid article id ({$id})");
            }
            
            //Populate form
            $arr = $row->toArray();
            $arr = $arr[0];
            $form->populate($arr);
            
            $this->view->row = $row;
        } else {
            $form->populate($_POST);
            //Invalid input data
            if (! $form->isValid($_POST)) {

            } else {
                
                //Disable view and layout
                $layout = Zend_Layout::getMvcInstance();
                if ($layout instanceof Zend_Layout) {
                    $layout->disableLayout();
                }
                
                $this->_helper->viewRenderer->setNoRender();
                $this->_table->editItem($id, $form);
                
                //Redirect to current controller
                $this->_helper->redirector->gotoRoute(array(
                    'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
                    'module' => 'default' ,
                    'controller' => $this->_controllerName , 
                    'module' => 'admin'
                ), null, true);
            }
        }
        $this->view->form = $form;
        $this->view->languages = Zend_Registry::get('config')->language->toArray();
    }

    /**
     * Called after rows fetch
     * 
     * @param $rows
     */
    protected function _fetchCallback ($rows)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        foreach ($rows as $row) {
            if ($row->active == 't') {
                $row->active = $translate->_('yes');
            } else {
                $row->active = $translate->_('no');
            }
        }
    }

    /**
     * Dispatch delete action
     * 
     * @return void
     */
    public function deleteAction ()
    {
        $result['replyCode'] = 200;
        
        //Input
        $request = $this->getRequest();
        $ids = $request->getParam('ids');
        
        //Create delele query
        $ids = explode(',', $ids);
        $ids = array_map('intval', $ids);
        $ids = 'id = ' . implode(' OR id = ', $ids);
        
        $this->_table->delete($ids);
        
        $this->_helper->json($result);
    }

    /**
     * Get add form
     * 
     * @return void
     */
    public function getAddForm ()
    {
        $form = new Giscuit_Form_Admin_Articles();
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'articlesForm' , 
            'action' => $this->_helper->url->url(array(
                'language' => Zend_Registry::get('Zend_Translate')->getLocale() ,
                'module' => 'admin' , 
                'controller' => $this->_controllerName , 
                'action' => 'add' , 
                'browse' => 'submit'
            ), null, true)
        ));
        return $form;
    }

    /**
     * Get edit form
     * 
     * @return void
     */
    public function getEditForm ($id)
    {
        $form = new Giscuit_Form_Admin_Articles();
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'articlesForm' , 
            'action' => $this->_helper->url->url(array(
                'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
                'module' => 'admin' , 
                'controller' => $this->_controllerName , 
                'action' => 'edit' , 
                'id' => $id , 
                'browse' => 'submit'
            ), null, true)
        ));
        
        return $form;
    }

}