<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Abstract.php 2844 2012-07-09 08:54:35Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Abstract class for Giscuit plugins
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Controller_Plugin_Abstract extends Zend_Controller_Plugin_Abstract
{
    /**
     * Name
     * 
     * @var string
     */
    public $name;
    
    /**
     * Settings
     * 
     * @var array
     */
    protected $_settings = array();
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) {
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) {
    }
    
    /**
     * Parse settings in form
     *  
     * @return boolean
     */
    public function parseSettingsForm($form) {
        return true;
    }
    
    /**
     * Parse settings in parameters
     *  
     * @return boolean
     */
    public function parseSettingsParams($config, $form) {
        return true;
    }
    
    /**
     * List of the javascript file to include
     * 
     * @return array
     */
    public function getJsFilesList() 
    {
        return array();
    }
    
    /**
     * List of the javascript file to include
     * 
     * @return array
     */
    public function getMobileJsFilesList() 
    {
        return array();
    }
    
    /**
     * Admin javascript script initialization code
     * 
     * @param $options Available options: 
     * @return string
     */
    public function getAdminJsInitCode(array $options = array()) 
    {
    }
    
    /**
     * Set settings
     * 
     * @param array $settings
     * @return void
     */
    public function setSettings(array $settings)
    {
        $this->_settings = $settings;    
    }
    
    /**
     * Get settings
     * 
     * @return array
     */
    public function getSettings()
    {
        return $this->_settings;
    }
    
    public static function arrayToTmx($locale, array $data = array())
    {
        //Header
        $tmx = "<?xml version='1.0' encoding='UTF-8' ?>\n<tmx version='1.4'>
<header segtype='sentence' o-tmf='unknown' adminlang='{$locale}' srclang='{$locale}'>
</header>\n<body>";

        //Body
        foreach($data as $key => $value) {
            if(isset($value)) {
                $escapedValue = htmlspecialchars($value);
                $tmx .= "<tu tuid='{$key}'>\n    <tuv xml:lang='{$locale}'><seg>{$escapedValue}</seg></tuv>\n</tu>\n";
            }
        }
        
        //Footer
        $tmx .= "</body>\n</tmx>";

        return $tmx;
    }

    /**
     * Get parameters
     * 
     * @return array
     */
    public function getParams() 
    {
        return array();
    }
    
    /**
     * Call the method $name in all defined functions 
     *
     * @param string $name
     * @param mixed $evt
     * 
     * @return array
     */
    public static function call($name, $evt)
    {
        $result = array();
        
        $front = Zend_Controller_Front::getInstance();
        $plugins = $front->getPlugins();
        foreach ($plugins as $plugin) {
            if(method_exists($plugin, $name)) {
                $result[$name] = $plugin->$name($evt);
            }
        }
        
        return $result;
    }
    
}

