<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for routing
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Routing extends Giscuit_Controller_Plugin_Abstract
{

    protected $_translations = array(
        'en' => array(
            'routing' => 'Routing' ,
            'noLayersAvailableForRouting' => 'No layers available for routing' ,
            'addDestination' => 'Add destination' ,
            'getDirections' => 'Get directions' ,
            'algorithms' => 'Algorithms' ,
            'costs' => 'Costs' ,
            'noRouteFound' => 'No route found' ,
            'destinationNotFound' => 'Destination not found' ,
            'getPositionFromMap' => 'Get position from map' ,
            'getPositionFromMyLocation' => 'Get position from my location' ,
            'shortestPathDijkstra' => 'Shortest Path (Dijkstra)' ,
            'shortestPathAStar' => 'Shortest Path (A-Star)' ,
            'shortestPathShootingStar' => 'Shortest Path (Shooting-Star)' ,
            'travelingSalesmanDijkstra' => 'Traveling Salesman (Dijkstra)' ,
            'travelingSalesmanAStar' => 'Traveling Salesman (A-Star)' ,
            'routingGeolocation' => 'Click on red point if the position is correct, otherwise click on map'
        ),
        'ro' => array(
            'routing' => 'Rutare' ,
            'noLayersAvailableForRouting' => 'Nu există straturi disponibile pentru rutare' ,
            'addDestination' => 'Adaugă destinaţie' ,
            'getDirections' => 'Căutarea rutei' ,
            'algorithms' => 'Algoritmi' ,
            'costs' => 'Costuri' ,
            'noRouteFound' => 'Ruta nu a fost găsită' ,
            'destinationNotFound' => 'Destinaţia nu a fost găsită' ,
            'getPositionFromMap' => 'Obţine poziţia de pe harta' ,
            'getPositionFromMyLocation' => 'Obţine poziţia din locaţia curentă' ,
            'shortestPathDijkstra' => 'Cel mai scurt drum (Dijkstra)' ,
            'shortestPathAStar' => 'Cel mai scurt drum (A-Star)' ,
            'shortestPathShootingStar' => 'Cel mai scurt drum (Shooting-Star)' ,
            'travelingSalesmanDijkstra' => 'Vînzător călător (Dijkstra)' ,
            'travelingSalesmanAStar' => 'Vînzător călător (A-Star)' ,
            'routingGeolocation' => 'Faceți click pe punct roșu daca va satisface precizia, sau faceți click oriunde pe harta'
        ),
        'ru' => array(
            'routing' => 'Маршруты' ,
            'noLayersAvailableForRouting' => 'Нет слоев для маршрутизации' ,
            'addDestination' => 'Добавить назначение' ,
            'getDirections' => 'Проложить маршрут' ,
            'algorithms' => 'Алгоритмы' ,
            'costs' => 'Затраты' ,
            'noRouteFound' => 'Маршрут не найден' ,
            'destinationNotFound' => 'Назначение не найдено' ,
            'getPositionFromMap' => 'Указать место на карте' ,
            'getPositionFromMyLocation' => 'Указать мое текущее местоположение' ,
            'shortestPathDijkstra' => 'Кратчайший путь (Dijkstra)' ,
            'shortestPathAStar' => 'Кратчайший путь (A-Star)' ,
            'shortestPathShootingStar' => 'Кратчайший путь (Shooting-Star)' ,
            'travelingSalesmanDijkstra' => 'Путешествующий торговец (Dijkstra)' ,
            'travelingSalesmanAStar' => 'Путешествующий торговец (A-Star)' ,
            'routingGeolocation' => 'Нажмите на красную точку если вас устраивает точность, или на любое место на карте'
        )
    );
    
    /**
     * List of the javascript files to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Routing.js' 
        );
    }
    
    /**
     * List of the javascript files to include for mobile
     * 
     * @return array
     */
    public function getMobileJsFilesList ()
    {
        return array(
            'Mobile/Control/Routing.js'
        );
    }
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) 
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $db = Zend_Registry::get('db');
        $rulesTable = new Giscuit_Db_Table_Rules();
        
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            $connectionString = Giscuit_Controller_Plugin_Init::getDbConnectionString();
            
            $pgsqlBasePath = realpath(GT_PGSQL_BIN_PATH . DIR_SEP . '..');
            $pgRoutingPath = GT_APP_PATH . DIR_SEP . '..' . DIR_SEP . '..' . DIR_SEP . '..' . DIR_SEP . 'pgRouting-1.03_pg-8.4.2';
            $routingCore = $pgRoutingPath . DIR_SEP . 'share' . DIR_SEP . 'contrib' . DIR_SEP . 'routing_core.sql' ;
            $routingCoreWrappers = $pgRoutingPath . DIR_SEP . 'share' . DIR_SEP . 'contrib' . DIR_SEP . 'routing_core_wrappers.sql' ;
            $routingTopology = $pgRoutingPath . DIR_SEP . 'share' . DIR_SEP . 'contrib' . DIR_SEP . 'routing_topology.sql';
            
            if (! file_exists($pgRoutingPath . DIR_SEP . 'lib' . DIR_SEP . 'librouting.dll')) {
                return $translate->_('Routing files are missing');
            }
            
            //Trgm
            $fileName = $pgsqlBasePath . DIR_SEP . 'share' . DIR_SEP . 'contrib' . DIR_SEP . 'pg_trgm.sql' ;
            Giscuit_Exec::exec('psql', "-q -d \"$connectionString\" -f \"$fileName\"", GT_PGSQL_BIN_PATH);
                
            //pgRouting
            copy(
                $pgRoutingPath . DIR_SEP . 'lib' . DIR_SEP . 'librouting.dll',
                $pgsqlBasePath . DIR_SEP . 'lib' . DIR_SEP . 'librouting.dll'
            );
            copy(
                $pgRoutingPath . DIR_SEP . 'lib' . DIR_SEP . 'librouting_dd.dll',
                $pgsqlBasePath . DIR_SEP . 'lib' . DIR_SEP . 'librouting_dd.dll'
            );
            copy(
                $pgRoutingPath . DIR_SEP . 'lib' . DIR_SEP . 'librouting_tsp.dll',
                $pgsqlBasePath . DIR_SEP . 'lib' . DIR_SEP . 'librouting_tsp.dll'
            );

            Giscuit_Exec::exec('psql', "-q -d \"$connectionString\" -f \"$routingCore\"", GT_PGSQL_BIN_PATH);
            Giscuit_Exec::exec('psql', "-q -d \"$connectionString\" -f \"$routingCoreWrappers\"", GT_PGSQL_BIN_PATH);
            Giscuit_Exec::exec('psql', "-q -d \"$connectionString\" -f \"$routingTopology\"", GT_PGSQL_BIN_PATH);
        }
        
        $data = array(
            'role' => 'guest',
            'resource' => 'tool_routing_controller',
            'privilege' => '',
            'group' => 'controller'
        );
        $rulesTable->insert($data);
        
        $schema = $newConfig->production->database->schema;
        $db->beginTransaction();
        try {
            $db->query("
                CREATE TABLE {$schema}.layer_routing (
                    id integer NOT NULL,
                    active character(1) DEFAULT 'f'::bpchar,
                    source pg_catalog.text,
                    reverse_source pg_catalog.text,
                    directed character(1) DEFAULT 'f'::bpchar,
                    reverse_cost character(1) DEFAULT 'f'::bpchar,
                    algorithm pg_catalog.text,
                    processing timestamp with time zone
                );");

            $db->query("ALTER TABLE ONLY {$schema}.layer_routing ADD CONSTRAINT layer_routing_pkey PRIMARY KEY (id);");
            $db->query("ALTER TABLE ONLY {$schema}.layer_routing ADD CONSTRAINT layer_routing_id_fkey FOREIGN KEY (id) " .
                "REFERENCES {$schema}.layers(id) ON DELETE CASCADE;");
            
            $db->query("
                CREATE TABLE {$schema}.routing_cost_columns (
                    id integer NOT NULL,
                    \"column\" pg_catalog.text,
                    reverse_column pg_catalog.text,
                    name_id integer,
                    layer_id integer
                );");
            
            $db->query("
                CREATE SEQUENCE {$schema}.routing_cost_columns_id_seq
                    START WITH 1
                    INCREMENT BY 1
                    NO MAXVALUE
                    NO MINVALUE
                    CACHE 1;");
            
            $db->query("ALTER SEQUENCE {$schema}.routing_cost_columns_id_seq OWNED BY {$schema}.routing_cost_columns.id;");
            $db->query("ALTER TABLE ONLY {$schema}.routing_cost_columns ALTER COLUMN id SET DEFAULT nextval('{$schema}.routing_cost_columns_id_seq'::regclass);");
            $db->query("ALTER TABLE ONLY {$schema}.routing_cost_columns ADD CONSTRAINT layer_routing_cost_coef_pkey PRIMARY KEY (id);");
            $db->query("ALTER TABLE ONLY {$schema}.routing_cost_columns ADD CONSTRAINT layer_routing_cost_coef_layer_id_fkey FOREIGN KEY (layer_id) " .
                "REFERENCES {$schema}.layers(id) ON DELETE CASCADE;");
            
            $db->query("
                CREATE OR REPLACE FUNCTION gt_assign_vertex_id(schema_name character varying, geom_table character varying, tolerance double precision, geo_cname character varying, gid_cname character varying)
                    RETURNS character varying AS
                \$BODY$
                DECLARE
                    _r record;
                    source_id int;
                    target_id int;
                    srid integer;
                    vert_geom_name text;
                BEGIN
                    vert_geom_name := quote_ident(schema_name) || '.' || quote_ident(geom_table) || '_vertices';
                    srid := Find_SRID(quote_ident(schema_name), quote_ident(geom_table), quote_ident(geo_cname));
                    
                    BEGIN
                    EXECUTE 'DROP TABLE ' || vert_geom_name;
                    EXCEPTION WHEN UNDEFINED_TABLE THEN
                    END;
                
                    EXECUTE 'CREATE TABLE ' || vert_geom_name || ' (gid serial)';
                    EXECUTE 'SELECT addGeometryColumn(''' || quote_ident(schema_name) || ''', ''' || quote_ident(geom_table) || '_vertices' || ''', ''the_geom'', '||srid||', ''POINT'', 2)';
                    EXECUTE 'CREATE INDEX ' || quote_ident(geom_table) || '_the_geom_gist ON ' || vert_geom_name || ' USING GIST (the_geom)';
                            
                    FOR _r IN EXECUTE 'SELECT ' || quote_ident(gid_cname) || ' AS gid,'
                        || ' ST_StartPoint('|| quote_ident(geo_cname) ||') AS source,'
                            || ' ST_EndPoint('|| quote_ident(geo_cname) ||') AS target'
                        || ' FROM ' || quote_ident(schema_name) || '.' || quote_ident(geom_table)
                        || ' WHERE ' || quote_ident(geo_cname) || ' IS NOT NULL '
                    LOOP
                        
                    source_id := gt_point_to_id(schema_name, geom_table, _r.source, tolerance);
                    target_id := gt_point_to_id(schema_name, geom_table, _r.target, tolerance);
                                                
                    EXECUTE 'UPDATE ' || quote_ident(schema_name) || '.' || quote_ident(geom_table) || 
                        ' SET source = ' || source_id || 
                        ', target = ' || target_id || 
                        ' WHERE ' || quote_ident(gid_cname) || ' =  ' || _r.gid;
                    END LOOP;
                    
                    RETURN 'OK';
                
                END;
                \$BODY$
                    LANGUAGE plpgsql VOLATILE STRICT;");

            $db->query("
                CREATE OR REPLACE FUNCTION gt_point_to_id(schema_name character varying, geom_table character varying, p geometry, tolerance double precision)
                RETURNS BIGINT 
                AS 
                $$ 
                
                DECLARE
                    _r record; 
                    _gid bigint; 
                    vert_geom_name text;
                BEGIN
                    vert_geom_name := quote_ident(schema_name) || '.' || quote_ident(geom_table) || '_vertices';
                
                    EXECUTE 'SELECT gid ' ||
                    'FROM ' || vert_geom_name || ' ' || 
                    'WHERE ST_DWithin(the_geom, $1, $2) ' ||
                    'ORDER BY ST_Distance(the_geom, $1) LIMIT 1'
                    INTO _r
                    USING p, tolerance;
                    
                    IF _r.gid IS NOT NULL THEN
                        _gid := _r.gid;
                    ELSE
                        EXECUTE 'INSERT INTO ' || vert_geom_name || ' (the_geom) VALUES ($1)' USING p;
                        _gid := lastval();
                    END IF;
                
                    RETURN _gid;
                
                END; $$ LANGUAGE 'plpgsql' VOLATILE STRICT;");
            
            $db->query("CREATE SCHEMA {$schema}_routing;");
            
            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
        
            $newMessages = array_merge($translate->getMessages(), $languageValue);
            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
            
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) 
    {
        $db = Zend_Registry::get('db');
        $rulesTable = new Giscuit_Db_Table_Rules();
        
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'tool_routing_controller');
        $rulesTable->delete($where);
        
        $schema = $newConfig->production->database->schema;
        $db->beginTransaction();
        try {
            $db->query("DROP TABLE IF EXISTS {$schema}.layer_routing, {$schema}.routing_cost_columns CASCADE;");
            $db->query("DROP FUNCTION IF EXISTS gt_assign_vertex_id(character varying, character varying, double precision, character varying, character varying);");
            $db->query("DROP FUNCTION IF EXISTS gt_point_to_id(character varying, character varying, geometry, double precision)");
            $db->query("DROP SCHEMA IF EXISTS {$schema}_routing CASCADE;");

            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
            
            $translations = $translate->getMessages();
            foreach ($languageValue as $key => $value) {
                unset($translations[$key]);
            }

            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $translations);
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Check layer type
     * 
     * @param string $layerName Layer name
     * @return boolean
     */
    public function isLayerLine($layerName)
    {
        if (class_exists($layerName) && 
            class_exists('Giscuit_Map_Layer_Line') && 
            is_subclass_of($layerName, 'Giscuit_Map_Layer_Line')) {
                
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Populate admin layers form
     * 
     * @param Zend_Form $form Layer form
     * @param Zend_Db_Row $options Row populated with layer data
     * @return void
     */
    public function adminLayersForm ($form, $options) 
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');
        
        //Active
        $form->addElement('checkbox', 'routing_active', array(
            'validators' => array(
                array(
                    'InArray' , 
                    false , 
                    array(
                        array(
                            't' , 
                            'f'
                        )
                    )
                ), 
                array(
                    'TableExists' , 
                    false , 
                    array(
                        array(
                            $config->database->schema , 
                            $options['table']
                        )
                    )
                )
            ) ,
            'required' => true , 
            'label' => 'active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Populate sources array
        $geocodeSources = array();
        foreach ($config->geocode->sources->toArray() as $source) {
            $geocodeSources[$source['id']] = $source['id'];
        }
        
        //FIXME:
        if (isset($geocodeSources['google'])) {
            unset($geocodeSources['google']);
        }
        
        //Source
        $form->addElement('select', 'routing_source', array(
            'required' => true , 
            'label' => 'source' , 
            'multiOptions' => $geocodeSources
        ));
        
        //Reverse source
        $form->addElement('select', 'routing_reverse_source', array(
            'required' => true , 
            'label' => 'reverse_source' , 
            'multiOptions' => $geocodeSources
        ));
        
        //Directed
        $form->addElement('checkbox', 'routing_directed', array(
            'validators' => $form->checkboxValidator ,
            'required' => true , 
            'label' => 'directed' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f' 
        ));
        
        //Reverse cost
        $form->addElement('checkbox', 'routing_reverse_cost', array(
            'validators' => $form->checkboxValidator ,
            'required' => true , 
            'label' => 'reverse_cost' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Algorithm
        $algorithmMultiOptions = Giscuit_Db_Table_LayerRouting::$algorithmNames;
        foreach ($algorithmMultiOptions as $key => $value) {
            $algorithmMultiOptions[$key] = $translate->_($value);
        }
        
        $form->addElement('multiCheckbox', 'routing_algorithm', array(
            'required' => true , 
            'label' => 'algorithm' , 
            'multiOptions' => $algorithmMultiOptions
        ));

        $form->addDisplayGroup(array(
            'routing_active' , 
            'routing_source' ,
            'routing_directed' ,
            'routing_reverse_cost' ,
            'routing_algorithm' 
        ), 'routing');
    }
    
    public function rebuildGraph ($layerId, $method)
    {
        $layersTable = new Giscuit_Db_Table_Layers();
        $routingTable = new Giscuit_Db_Table_LayerRouting();
        $config = Zend_Registry::get('config');
        $schema = $config->database->schema;
        
        //Check if routing row exists and create one if it doesn't
        $routingRow = $routingTable->find($layerId)->current();
        if ($routingRow == NULL) {
            $routingRow = $routingTable->createRow();
            $routingRow->id = $layerId;
            $routingRow->save();
        }
        
        //Get the layer's table name
        $layer = $layersTable->getItems(array('whereIds' => array($layerId)));
        $timestamp = 'layer_' . $layerId . '_' . time();

        //Create sql text
        $sqlText = "BEGIN;\n";
        $sqlText .= "SET client_min_messages TO WARNING;\n";
        $sqlText .= $this->adminLayersCreateTableSql($timestamp, $layer[$layerId]['table']);
        if ($method == 1) {
            $sqlText .= $this->adminLayersCreateOneToOneSql($timestamp, $layer[$layerId]['table']);
        } else {
            $sqlText .= $this->adminLayersCreateOneToManySql($timestamp, $layer[$layerId]['table']);
        }
        $sqlText .= "UPDATE {$schema}.layer_routing SET processing = null WHERE id={$layerId};\n";
        $sqlText .= "COMMIT;\n";
        
        $connectionString = Giscuit_Controller_Plugin_Init::getDbConnectionString();
        $commands = array();
        
        //Create executable commands and necessary files
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            $psqlCmd = '"' . realpath(GT_PGSQL_BIN_PATH) . DIR_SEP . 'psql.exe"';
            $scriptExt = '.bat';
            $scriptJoin = "\r\n";
            $scriptPrefix = 'CALL ';
            
            $commands[] = '@ECHO OFF';
        } else {
            $psqlCmd = 'psql';
            $scriptExt = '.sh';
            $scriptJoin = "\n";
            $scriptPrefix = '';
            
            $commands[] = '#!/bin/sh';
        }
        
        $filename = 'routing_' . $layerId;
        $graphScriptFile = GT_TMP_PATH . DIR_SEP . $filename . $scriptExt;
        $graphSqlFile = GT_TMP_PATH . DIR_SEP . $filename . '.sql';
        $graphOutputFile = GT_TMP_PATH . DIR_SEP . $filename . '.log';

        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -f "' . $graphSqlFile . '"';
        $commands[] = 'exit';
        
        @unlink($graphScriptFile);
        @unlink($graphSqlFile);
        @unlink($graphOutputFile);
        
        file_put_contents($graphScriptFile, implode($scriptJoin, $commands));
        file_put_contents($graphSqlFile, $sqlText);
        chmod($graphScriptFile, 0700);

        //Start the graph building action
        $updateProcess = array(
            'processing' => new Zend_Db_Expr('now()')
        );
        $where = $routingTable->getAdapter()->quoteInto('id = ?', $layerId);
        $routingTable->update($updateProcess, $where);

        Giscuit_Exec::background($scriptPrefix . $graphScriptFile, $graphOutputFile);
    }
    
    public function adminLayersCreateTableSql ($timestamp, $tableName)
    {
        $config = Zend_Registry::get('config');
        $schema = $config->database->schema;
        $srid = $config->map->srid;
        
        return "
            CREATE TABLE {$schema}_routing.\"{$timestamp}\" (
               gid serial,
               parent_gid integer,
               source integer,
               target integer,
               cost double precision,
               reverse_cost double precision,
               x1 double precision,
               y1 double precision,
               x2 double precision,
               y2 double precision,
               to_cost double precision,
               rule text
            );
            
            SELECT AddGeometryColumn('{$schema}_routing', '{$timestamp}', 'the_geom', {$srid}, 'MULTILINESTRING', 2);
            
            ALTER TABLE ONLY {$schema}_routing.\"{$timestamp}\" ADD CONSTRAINT \"{$timestamp}_pkey\" PRIMARY KEY (gid);
            
            CREATE INDEX \"{$timestamp}_gid_btree\" ON {$schema}_routing.\"{$timestamp}\" USING btree (gid);
            
            CREATE INDEX \"{$timestamp}_source_btree\" ON {$schema}_routing.\"{$timestamp}\" USING btree (source);
            
            CREATE INDEX \"{$timestamp}_target_btree\" ON {$schema}_routing.\"{$timestamp}\" USING btree (target);
        ";
    }
    
    public function adminLayersCreateOneToOneSql ($timestamp, $tableName)
    {
        $config = Zend_Registry::get('config');
        $schema = $config->database->schema;
        
        if ($config->map->units == 'dd') {
            $snapSize = "0.00001";
            $useSpheroid = ', true';
        } else {
            $snapSize = "1.11";
            $useSpheroid = '';
        }
        
        return "
            INSERT INTO {$schema}_routing.\"{$timestamp}\" (SELECT
            gid,
            gid as parent_gid,
            NULL as source,
            NULL as target,
            ST_Length(the_geom{$useSpheroid}) as cost,
            ST_Length(the_geom{$useSpheroid}) as reverse_cost,
            x(ST_StartPoint(the_geom)) as x1,
            y(ST_StartPoint(the_geom)) as y1,
            x(ST_EndPoint(the_geom)) as x2,
            y(ST_EndPoint(the_geom)) as y2,
            NULL as to_cost,
            NULL as rule,
            the_geom
            FROM {$schema}_layers.\"{$tableName}\");
            
            SELECT gt_assign_vertex_id('{$schema}_routing', '{$timestamp}', {$snapSize}, 'the_geom', 'gid');
            
            DROP TABLE IF EXISTS {$schema}_routing.\"{$tableName}\", {$schema}_routing.\"{$tableName}_vertices\" CASCADE;
            
            ALTER TABLE {$schema}_routing.\"{$timestamp}\" RENAME TO \"{$tableName}\";
            ALTER TABLE {$schema}_routing.\"{$timestamp}_vertices\" RENAME TO \"{$tableName}_vertices\";
            
            DELETE FROM public.geometry_columns WHERE f_table_name = '{$timestamp}';
            DELETE FROM public.geometry_columns WHERE f_table_name = '{$timestamp}_vertices';
        ";
    }
    
    public function adminLayersCreateOneToManySql ($timestamp, $tableName)
    {
        $config = Zend_Registry::get('config');
        $schema = $config->database->schema;
        
        if ($config->map->units == 'dd') {
            $snapSize = "0.00001";
            $useSpheroid = ', true';
        } else {
            $snapSize = "1.11";
            $useSpheroid = '';
        }
        
        return "
            INSERT INTO {$schema}_routing.\"{$timestamp}\" (
            SELECT DISTINCT ON (the_geom)
            nextval('{$schema}_routing.{$timestamp}_gid_seq') AS gid,
            gid AS parent_gid,
            NULL as source,
            NULL as target,
            ST_Length(part_geom{$useSpheroid}) as cost,
            ST_Length(part_geom{$useSpheroid}) as reverse_cost,
            x(ST_StartPoint(part_geom)) as x1,
            y(ST_StartPoint(part_geom)) as y1,
            x(ST_EndPoint(part_geom)) as x2,
            y(ST_EndPoint(part_geom)) as y2,
            NULL as to_cost,
            NULL as rule,
            ST_Multi(part_geom) AS the_geom
            FROM (
               SELECT a.gid, a.the_geom, (ST_Dump(ST_Union(b.the_geom))).geom AS part_geom
               FROM {$schema}_layers.\"{$tableName}\" as a, {$schema}_layers.\"{$tableName}\" as b
               WHERE ST_Intersects(a.the_geom, b.the_geom)
               GROUP BY a.gid, a.the_geom
            ) AS res
            WHERE ST_Contains(the_geom, part_geom) = true OR ST_Contains(ST_Buffer(the_geom, {$snapSize}), part_geom) = true);
            
            SELECT gt_assign_vertex_id('{$schema}_routing', '{$timestamp}', {$snapSize}, 'the_geom', 'gid');
            
            DROP TABLE IF EXISTS {$schema}_routing.\"{$tableName}\", {$schema}_routing.\"{$tableName}_vertices\" CASCADE;
            
            ALTER TABLE {$schema}_routing.\"{$timestamp}\" RENAME TO \"{$tableName}\";
            ALTER TABLE {$schema}_routing.\"{$timestamp}_vertices\" RENAME TO \"{$tableName}_vertices\";
            
            DELETE FROM public.geometry_columns WHERE f_table_name = '{$timestamp}';
            DELETE FROM public.geometry_columns WHERE f_table_name = '{$timestamp}_vertices';
        ";
    }

}