<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: JobsActions.php 2894 2012-08-01 14:23:23Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for Jobs table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_JobsActions extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'jobs_actions';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>jobIds string - Comma separated job ids whose actions are fetched</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'user_id' ,
            'job_id' , 
            'action_map_id' , 
            'comment' , 
            'date' => "extract(epoch from date)" , 
            'the_geom' => 'ST_AsText(the_geom)'
        );
        
        $select->from($this, $fields);
        
        $select->where($this->_name . '.job_id IN ' . $options['jobIds']);
        
        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Add item
     * 
     * @param array $data Action data
     * @return integer The id of the new element
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        
        $newRow->user_id = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $newRow->job_id = $data['jobId'];
        $newRow->action_map_id = $data['action'];
        $newRow->comment = $data['comment'];;
        $newRow->date = new Zend_Db_Expr('now()');
        $newRow->the_geom = new Zend_Db_Expr("ST_MakePoint({$data['coordX']}, {$data['coordY']})");

        $result = $newRow->save();

        return $result;
    }
}
