<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Sessions.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for sessions table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Sessions extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'sessions';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'user_id';

    /**
     * Current user id
     * 
     * @var integer
     */
    protected $_userId;

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $this->_userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if (empty($this->_userId)) {
            throw new Giscuit_Db_Table_Exception("Invalid user id");
        }
    }

    /**
     * Initialize the session
     * 
     * @return void
     */
    public function start ()
    {
        $result = $this->get();
        
        if (empty($result)) {
            $this->add();
            $this->get();
        } else {
            $this->edit();
        }
    
    }

    /**
     * Get the current session
     * 
     * @return Zend_Db_Table_Rowset
     */
    public function get ()
    {
        
        $select = $this->select($this, array(
            'session'
        ));
        $select->setIntegrityCheck(false);
        
        $select->where($this->_name . '.user_id = ?', $this->_userId);
        
        $result = $this->fetchRow($select);
        
        if (empty($result->session)) {
            return;
        }
        
        return $result;
    }

    /**
     * Add session
     * 
     * @return integer The id of the new element
     */
    public function add ()
    {
        $newRow = $this->createRow();
        
        $newRow->user_id = $this->_userId;
        $newRow->session = Zend_Session::getId();
        $newRow->date = new Zend_Db_Expr('now()');
        
        return $newRow->save();
    }

    /**
     * Update the current session
     * 
     * @return void
     */
    public function edit ()
    {
        $rows = $this->find($this->_userId);
        $row = $rows->current();
        $row->session = Zend_Session::getId();
        $row->date = new Zend_Db_Expr('now()');
        
        $row->save();
    }
}
