<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Jobs.php 2182 2012-01-20 12:01:13Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Jobs layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Jobs extends Giscuit_Form_Layer_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        $acl = Zend_Registry::get('acl');
        
        //Set method and id
        $this->setMethod('post');
        $this->addAttribs(array(
            'id' => 'gtJobsForm'
        ));
        
        //Title
        $this->addElement('text', 'title', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) ,  
            'required' => true , 
            'label' => 'title',
            'order' => 1
        ));
        
        //Description
        $this->addElement('textarea', 'desc', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) , 
            'required' => true , 
            'rows' => 5,
            'cols' => 40,
            'label' => 'description',
            'order' => 2
        ));
        
        //Workflow
        $workflowsTable = new Giscuit_Db_Table_JobsActionsWorkflows();
        $workflows = $workflowsTable->getItems();
        $workflowsArr = array();
        foreach ($workflows as $workflow) {
            $workflowsArr[$workflow->id] = $workflow->name;
        }
        $this->addElement('select', 'workflow', array(
            'required' => true , 
            'label' => 'workflow' , 
            'multiOptions' => $workflowsArr,
            'order' => 3
        ));
        
        //To user
        $usersTable = new Giscuit_Db_Table_Users();
        $users = $usersTable->getItems(array(
            'withIdentity' => true,
            'whereActive' => 't'
        ));
        $jobsTable = new Giscuit_Db_Table_Jobs();
        $activeJobs = $jobsTable->getItems(array(
            'whereStatus' => "('pending', 'in_progress')"
        ));
        $usersArr = array();
        foreach ($users as $user) {
            if ($acl->inheritsRole($user->group, 'jobs_worker') || $user->group == 'admin') {
                $userRecord['id'] = $user->id;
                $userRecord['identity'] = $user->identity;
                $userRecord['jobsAmount'] = 0;
                $userRecord['jobs'] = array();
                foreach ($activeJobs as $job) {
                    if ($job['to_user_id'] == $user->id) {
                        $userRecord['jobsAmount']++;
                        $userRecord['jobs'][] = $job['title'];
                    }
                }
                $usersArr[] = $userRecord;
            }
        }
        
        $options = array();
        $disabledOptions = array();
        foreach ($usersArr as $user) {
            $options[$user['id']] = $user['identity'] . ' (' . $user['jobsAmount'] . ' ' . $translate->_('tasks') . ')';
            foreach ($user['jobs'] as $job) {
                $key = '_' . $job;
                $options[$key] = html_entity_decode('&nbsp;&nbsp;&nbsp;&nbsp;', ENT_COMPAT, 'UTF-8') . $job;
                $disabledOptions[] = $key;
            }
        }
        $this->addElement('select', 'to_user', array(
            'required' => true , 
            'label' => 'assignTo' , 
            'multiOptions' => $options,
            'disable' => $disabledOptions,
            'order' => 4
        ));
        
        //Priority
        $priority = array(
            'low' => 'low',
            'medium' => 'medium',
            'high' => 'high'
        );
        $this->addElement('select', 'priority', array(
            'required' => true , 
            'label' => 'priority' , 
            'multiOptions' => $priority,
            'order' => 6
        ));
        
        //Deadline
        $this->addElement('date', 'deadline', array(
            'label' => 'deadline' , 
            'attribs' => array(
                'id' => 'gtJobsDeadline'
            ),
            'order' => 7
        ));
        
        //Notify
        $notifyArr = array(
            '1' => $translate->_('dontNotify'),
            '2' => $translate->_('notifyActions'),
            '3' => $translate->_('notifyComplete')
        );
        $this->addElement('radio', 'notify', array(
            'required' => true , 
            'label' => 'notify' , 
            'multiOptions' => $notifyArr,
            'value' => 1,
            'order' => 8
        ));
        
        //Contact address
        $this->addElement('text', 'contact_address', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) ,  
            'required' => false , 
            'label' => 'contact_address',
            'order' => 9
        ));
        
        //Contact name
        $this->addElement('text', 'contact_name', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) ,  
            'required' => false , 
            'label' => 'contact_name',
            'order' => 10
        ));
        
        //Contact phone
        $this->addElement('text', 'contact_phone', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) ,  
            'validators' => array(
                'Phone'
            ) , 
            'required' => false , 
            'label' => 'contact_phone',
            'order' => 11
        ));
        
    }
    
    public function addRevertField ()
    {
        //Revert jobs status
        $this->addElement('checkbox', 'revert', array(
            'validators' =>  $this->checkboxValidator ,
            'required' => true , 
            'label' => 'revertStatusPending' ,
            'checkedValue' => 't' , 
            'uncheckedValue' => 'f',
            'order' => 5
        ));
    }

}