<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Vector.php 2095 2011-12-14 12:05:50Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Map_Layer_Abstract
 */
require_once 'Giscuit/Map/Layer/Abstract.php';

/**
 * Class for abstract mapserver wfs layer object
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Map_Layer_Wfs extends Giscuit_Map_Layer_Vector
{
    /**
     * Request timeout
     * 
     * @var string
     */
    public static $timeout = 10;
    
    public static function createConvertScript($url, $srcName, $dstName, $geometryType, array $auth = array(), $version) 
    {
        $config = Zend_Registry::get('config');
        
        $commands = array();
        $schema = $config->database->schema . '_layers';
        $connectionString = Giscuit_Controller_Plugin_Init::getDbConnectionString();
        
        //
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            $psqlCmd = '"' . realpath(GT_PGSQL_BIN_PATH) . DIR_SEP . 'psql.exe"';
            $ogr2ogrCmd = '"' . realpath(GT_GDAL_BIN_PATH) . DIR_SEP . 'ogr2ogr.exe"';
            $nullCmd = 'NUL';
            $scriptExt = '.bat';
            $scriptJoin = "\r\n";
        } else {
            $psqlCmd = 'psql';
            $ogr2ogrCmd = 'ogr2ogr';
            $nullCmd = '/dev/null';
            $scriptExt = '.sh';
            $scriptJoin = "\n";
            
            $commands[] = '#!/bin/sh';
        }
        
        //
        $wfsConnectionFile = GT_TMP_PATH . DIR_SEP . $dstName . '.xml';
        self::createVirtualWfs($wfsConnectionFile, $url, $auth, $version);    

        /*
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . ' DROP CONSTRAINT enforce_dims_the_geom" > ' . $nullCmd . ' 2>&1';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . ' DROP CONSTRAINT enforce_geotype_the_geom" > ' . $nullCmd . ' 2>&1';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . ' DROP CONSTRAINT enforce_srid_the_geom" > ' . $nullCmd . ' 2>&1';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . ' DROP CONSTRAINT ' . $dstName . '_tmp_pk" > ' . $nullCmd . ' 2>&1';
        */
        
        $commands[] = $ogr2ogrCmd . ' -f "PostGreSQL" ' .
            '-nln "' . $dstName . '_tmp" -nlt ' . $geometryType . ' -skipfailures -gt 1024 -t_srs "EPSG:' . $config->map->srid . '" -lco GEOMETRY_NAME="the_geom" -lco SCHEMA="' . $schema . '" -lco OVERWRITE=YES --config PG_USE_COPY YES ' .
            'PG:"' . $connectionString . '" ' .
            '"' . $wfsConnectionFile . '" "'  . $srcName . '"';
            
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . '_tmp DROP COLUMN gid CASCADE" > ' . $nullCmd . ' 2>&1';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . '_tmp DROP COLUMN gml_id CASCADE" > ' . $nullCmd . ' 2>&1';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . '_tmp RENAME ogc_fid TO gid"';
        
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "DROP TABLE IF EXISTS ' . $schema . '.' . $dstName . ' CASCADE"';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . '_tmp RENAME TO ' . $dstName . '"';
            
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER INDEX ' . $schema . '.' . $dstName . '_tmp_pk RENAME TO ' . $dstName . '_pkey"';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER INDEX ' . $schema . '.' . $dstName . '_tmp_geom_idx RENAME TO ' . $dstName . '_geom_idx"';
    
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . '_tmp_ogc_fid_seq RENAME TO ' . $dstName . '_gid_seq"';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "ALTER TABLE ' . $schema . '.' . $dstName . ' ALTER COLUMN gid SET DEFAULT NEXTVAL(\'' . $schema . '.' . $dstName . '_gid_seq\')"';
            
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "DELETE FROM public.geometry_columns WHERE f_table_name = \'' . $dstName . '\' AND f_table_schema = \'' . $schema . '\'"';
        $commands[] = $psqlCmd . ' -q -d "' . $connectionString . '" -c "UPDATE public.geometry_columns SET f_table_name = \'' . $dstName . '\' WHERE f_table_name = \'' . $dstName . '_tmp\' AND f_table_schema = \'' . $schema . '\'"';
        
        //
        $wfsScriptFile = GT_TMP_PATH . DIR_SEP . $dstName . $scriptExt;
        file_put_contents($wfsScriptFile, implode($scriptJoin, $commands));
        chmod($wfsScriptFile, 0700);
        
        return true;
    }
    
    public static function getGeometryType($layer, $url, $auth, $version) 
    {
        //
        $url = parse_url($url);
        $query = 'MAXFEATURES=1';
        if(empty($url['query'])) {
            $url['query'] = $query;
        } else {
            $url['query'] .= '&' . $query;
        }
        $url = self::unparse_url($url);
        
        //
        $wfsConnectionFile = tempnam(GT_TMP_PATH, 'gt_');
        self::createVirtualWfs($wfsConnectionFile, $url, $auth, $version);
        $result = Giscuit_Exec::exec('ogrinfo', " -ro -q -fields=NO \"{$wfsConnectionFile}\" \"{$layer}\"", GT_GDAL_BIN_PATH);
        unlink($wfsConnectionFile);
        
        if(! empty($result['error'])) {
            return 'GEOMETRY';
        } else {
            if(stristr($result['output'], 'point')) {
                return 'POINT';
            } else if(stristr($result['output'], 'line')) {
                return 'MULTILINESTRING';
            } else if(stristr($result['output'], 'polygon')) {
                return 'MULTIPOLYGON';
            } else {
                return 'GEOMETRY';
            }
        }
    }
    
    public static function createVirtualWfs($path, $url, $auth, $version) 
    {
        $timeout = self::$timeout;
            
        $authContent = '';
        if(! empty($auth['type'])) {
            $authContent .= '<HttpAuth>' . strtoupper($auth['type']) . '</HttpAuth>' .
            $authContent .= "<UserPwd>{$auth['username']}:{$auth['password']}</UserPwd>";
        }
            
        $wfsConnectionContent = '<OGRWFSDataSource>' .
            "<URL>{$url}</URL>" .
            "<Timeout>{$timeout}</Timeout>" .
            $authContent .
            "<Version>{$version}</Version>" .
        '</OGRWFSDataSource>';
        
        file_put_contents($path, $wfsConnectionContent);        
    }

    public static function unparse_url($parsed_url) { 
          $scheme   = isset($parsed_url['scheme']) ? $parsed_url['scheme'] . '://' : ''; 
          $host     = isset($parsed_url['host']) ? $parsed_url['host'] : ''; 
          $port     = isset($parsed_url['port']) ? ':' . $parsed_url['port'] : ''; 
          $user     = isset($parsed_url['user']) ? $parsed_url['user'] : ''; 
          $pass     = isset($parsed_url['pass']) ? ':' . $parsed_url['pass']  : ''; 
          $pass     = ($user || $pass) ? "$pass@" : ''; 
          $path     = isset($parsed_url['path']) ? $parsed_url['path'] : ''; 
          $query    = isset($parsed_url['query']) ? '?' . $parsed_url['query'] : ''; 
          $fragment = isset($parsed_url['fragment']) ? '#' . $parsed_url['fragment'] : '';
           
          return "$scheme$user$pass$host$port$path$query$fragment"; 
    }

}
