<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Menu.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Abstract class for hierarchic menus
 * 
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Menu
{
    /**
     * Build menu
     * 
     * @param array $data
     * @return string
     */
    abstract public function buildMenu (array &$data);
    
    /**
     * Build menu element
     * 
     * @param array $row Element
     * @return string
     */
    abstract protected function _buildRow ($row);
    
    /**
     * Formats menu element
     * 
     * @param array $child Child element
     * @return array
     */
    abstract protected function _parse ($child);

    /**
     * Filter
     * 
     * @var integer
     */
    protected $_filter = 0;

    /**
     * Holds the data representing the menu
     * 
     * @var array
     */
    protected $_arr = array();

    /**
     * Id field name
     * 
     * @var string
     */
    protected $_idField = 'id';

    /**
     * Parent id field name
     * 
     * @var string
     */
    protected $_parentIdField = 'parent_id';

    /**
     * Holds the 2d version of the menu
     * 
     * @var array
     */
    protected $_data;

    /**
     * Filter
     * 
     * @param array $row
     * @return boolean
     */
    protected function _filterMethod ($row)
    {
        $parentId = $row[$this->_parentIdField];
        if(intval($parentId) <= 0) {
            $parentId = NULL;    
        }
        
        return $parentId == $this->_filter;
    }
    
    /**
     * Create a nested menu from 2d array data
     * 
     * @param array $data
     * @param array $arr
     * @param array $parent
     * @return array
     */
    protected function _buildNestedArray (&$data, &$arr, $parent)
    {
        
        $index = 0;
        $this->_filter = $parent;
        $children = array_filter($data, array(
            $this , 
            '_filterMethod'
        ));
        
        foreach ($children as $child) {
            $child = $this->_parse($child);
            $arr[$index] = $child;
            $this->_buildNestedArray($data, $arr[$index]['child'], $child[$this->_idField]);
            $index ++;
        }
    }

    /**
     * Initialize the creation of a nested menu from 2d array data
     * 
     * @param array $data
     * @return array
     */
    public function buildNestedArray (array $data)
    {
        if(! empty($data)) {
            $this->_data = $data;
            $this->_buildNestedArray($this->_data, $this->_arr, 0);
        }
        
        return $this->_arr; 
    }
    
    /**
     * Call buildMenu method
     * 
     * @return string
     */
    public function __toString ()
    {
        return $this->buildMenu($this->_arr);
    }

    /**
     * Set id field name
     * 
     * @param string $name
     * @return void
     */
    public function setIdField ($name)
    {
        $this->_idField = $name;
    }

    /**
     * Get id field name
     * 
     * @return string
     */
    public function getIdField ()
    {
        return $this->_idField;
    }

    /**
     * Set parent id field name
     * 
     * @param string $name
     * @return void
     */
    public function setParentIdField ($name)
    {
        $this->_parentIdField = $name;
    }

    /**
     * Get parent id field name
     * 
     * @return string
     */
    public function getParentIdField ()
    {
        return $this->_parentIdField;
    }
    
}