<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Navigation.php 1346 2011-05-24 19:28:03Z ue $
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for hierarchic navigation menu
 * 
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Menu_Mobile_Navigation extends Giscuit_Menu
{
    /**
     * 
     * 
     * @var string
     */
    protected $_baseUrl;

    /**
     * Build a menu using html lists template
     * 
     * @param array $data
     * @return string
     */
    public function buildMenu (array &$data)
    {
        if (empty($data)) {
            return '';
        }
        
        if(! $data[0]['parent_id']) {
            $html = '<ul data-role="listview">';
        } else {
            $html = "<ul>";
        }
        
        foreach ($data as $row) {
            $html .= $this->_buildRow($row);
            if (! empty($row['child'])) {
                $html .= $this->buildMenu($row['child']) . "</li>";
            } else {
                $html .= "</li>";
            }
        }
        
        $html .= "</ul>";
        
        return $html;
    }

    /**
     * Build menu element
     * 
     * @param array $row Element
     * @return string
     */
    protected function _buildRow ($row)
    {
        $attrs = '';
                
        if($row['url'] == $this->_baseUrl . '/auth/logout/jqm/1' ||
            $row['url'] == $this->_baseUrl . '/map' ||
            $row['url'] == $this->_baseUrl . '/auth'
            ) {
            $attrs .= ' rel=external';
        }
        
        $text = "<li><a href=\"{$row['url']}\"{$attrs}>{$row['title']}</a>";
        
        return $text;
    }
    
    /**
     * Formats menu elements
     * 
     * @param array $child Elnewsement of the menu
     * @return array
     */
    protected function _parse ($child)
    {
        $locale = Zend_Registry::get('Zend_Translate')->getLocale();
        
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');

        if (! empty($child['type'])) {
            $voidUrl = "javascript:void(0);";
            
            if ($child['type'] == 1) {
                $child['url'] = $voidUrl;
            } elseif ($child['type'] == 2) {
                $child['url'] = $url->url(array(
                    'language' => $locale , 
                    'module' => 'mobile' ,
                    'controller' => 'menu' , 
                    'action' => 'browse' , 
                    'id' => $child['id']
                ), null, true);
            } else {
                $child['url'] = str_replace('<language>', $locale, $child['url']);
                $child['url'] = str_replace('<front>', $this->_baseUrl, $child['url']);
            }
        }
        
        if (! empty($child['url'])) {
            $child['url'] = htmlspecialchars($child['url'], ENT_QUOTES);
        }
        $child['title'] = htmlspecialchars($child['title']);
        
        return $child;
    }
    
    /**
     * Initialize the creation of a nested menu from 2d array data
     * 
     * @param array $data
     * @return array
     */
    public function buildNestedArray (array $data)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $locale = Zend_Registry::get('Zend_Translate')->getLocale();
         
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $this->_baseUrl = $url->url(array(
            'language' =>  $locale,
            'module' => 'mobile'
        ), null, true);
        
        if (! empty($userId)) {
            array_push($data, array(
                'id' => 65533 , 
                'parent_id' => 0 , 
                'type' => 3 , 
                'url' => '<front>/auth/logout/jqm/1' ,
                'title' => $translate->_('logout')
            ));
        } else {
            array_push($data, array(
                'id' => 65534 , 
                'parent_id' => 0 , 
                'type' => 3 , 
                'url' => '<front>/auth' , 
                'title' => $translate->_('login')
            ));
            array_push($data, array(
                'id' => 65535 , 
                'parent_id' => 0 , 
                'type' => 3 , 
                'url' => '<front>/auth/signup', 
                'title' => $translate->_('signup')
            ));
        }
        
        $this->_data = $data;
        $this->_buildNestedArray($this->_data, $this->_arr, 0);
        
        
        return $this->_arr; 
    }

}