/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Celleditor.js 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: YAHOO.widget.ColorCellEditor
 * 
 */
(function () {

    var lang = YAHOO.lang,
        util = YAHOO.util,
        widget = YAHOO.widget,
        ua = YAHOO.env.ua,
        Dom = util.Dom,
        Ev = util.Event,
        DT = widget.DataTable;

    var BCE = widget.BaseCellEditor;

    widget.ColorCellEditor = function (oConfigs) {
        this._sId = "yui-colorceditor" + YAHOO.widget.BaseCellEditor._nCount++;
        widget.ColorCellEditor.superclass.constructor.call(this, "color", oConfigs);
    };

    lang.extend(widget.ColorCellEditor, BCE, {

        colorPicker: null,

        defaultColor: '000000',

        /**
         * Render a colorPicker.
         *
         * @method renderForm
         */
        renderForm: function () {
            if (this.disableBtns) {
                this.handleDisabledBtns();
            }
        },

        /**
         * After rendering form, if disabledBtns is set to true, then sets up a mechanism
         * to save input without them. 
         *
         * @method handleDisabledBtns
         */
        handleDisabledBtns: function () {

        },

        /**
         * Resets ColorCellEditor UI to initial state.
         *
         * @method resetForm
         */
        resetForm: function () {
            if (YAHOO.widget.ColorPicker == undefined) {
                return;
            }

            if(this.value) {
                var value = this.value.substr(1);
            } else {
                var value = this.defaultColor;
            }

            //FIXME: Do not destroy color picker, used the initialized one
            if (this.colorContainer != undefined && this.colorSetContainer != undefined) {
                this.getContainerEl().removeChild(this.colorContainer);
                this.colorPicker = null;
                
                YAHOO.util.Event.purgeElement(this.colorSetContainer, true); 
                this.getContainerEl().removeChild(this.colorSetContainer);
            }

            this.colorContainer = document.createElement("div");
            YAHOO.util.Dom.insertBefore(this.colorContainer, this.getContainerEl().firstChild);
            
            this.colorContainer.className = "color-picker-container";
            this.getContainerEl().style.width = '210px';

            this.colorPicker = new YAHOO.widget.ColorPicker(this.colorContainer, {
                showcontrols: false,
                images: {
                    PICKER_THUMB: "http://yui.yahooapis.com/2.8.0r4/build/colorpicker/assets/picker_thumb.png",
                    HUE_THUMB: "http://yui.yahooapis.com/2.8.0r4/build/colorpicker/assets/hue_thumb.png"
                }
            });

            this.colorPicker.setValue(YAHOO.util.Color.hex2rgb(value));
                        
            this.colorSetContainer = document.createElement("div");
            this.colorSetContainer.style.paddingTop = '185px';
            YAHOO.util.Dom.insertBefore(this.colorSetContainer, this.getContainerEl().firstChild.nextSibling);
            
            //Input
            this.colorSet = this.colorSetContainer.appendChild(document.createElement("input"));
            this.colorSet.style.width = '60px';
            this.colorSet.value = '#' + value;
            
            this.colorPicker.on("rgbChange", function(o) {
                this.colorSet.value = '#' + this.colorPicker.get("hex");
            }, null, this);

        },

        /**
         * Sets focus in ColorCellEditor.
         *
         * @method focus
         */
        focus: function () {

        },

        /**
         * Retrieves input value from ColorCellEditor.
         *
         * @method getInputValue
         */
        getInputValue: function () {
            var reg = /^\#([0-9a-f]){6}$/i;
            if(reg.test(this.colorSet.value) == false) {
                this.colorSet.value = '';
            }
           
            return this.colorSet.value;
        }

    });

    // Copy static members to ColorCellEditor class
    lang.augmentObject(widget.ColorCellEditor, BCE);

})();