/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Control.js 3401 2013-05-14 17:25:57Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Control
 * 
 *  Inherits from:
 *   - <OpenLayers.Control>
 */
Giscuit.Control = OpenLayers.Class(OpenLayers.Control, {

    callbackTimeout: 10000,

    callbackFailure: function(o) {
        Giscuit.Util.callbackFailure(o);
    },
    
    panel: null,

    initialize: function (options) {
        debug.info('Initializing control ' + this.CLASS_NAME);
        
        OpenLayers.Control.prototype.initialize.apply(this, [options]);
    },

    activate: function () {
        debug.info('Activating control ' + this.CLASS_NAME);
        
        return OpenLayers.Control.prototype.activate.call(this);
    },

    deactivate: function () {
        return OpenLayers.Control.prototype.deactivate.call(this);
    },
    
    clear: function() {
        if(this.active) {
            this.deactivate();
        }
        
        if(this.panel) {
             gt.accord.removePanel(this.getPanelIndex());
        }
        this.panel = null;
        
        debug.info('Successfully cleared ' + this.CLASS_NAME + ' control');
    },

    getLoadingImage: function () {
        return "<img src='" + gt.config.httpImagePath + "/gt-ajax-loader.gif'>";
    },
    
    getPanelIndex: function (panel) {
        //
        if(typeof(panel) == 'undefined') {
            panel = this.panel;
        }
        
        //
        if(panel === null) {
            return false;
        }
        
        //
        for(var i=0; i < panel.parentNode.children.length; i++) {
            var el = panel.parentNode.children[i];
            
            if(el == panel) {
                return i;
            }
        }
        
        return false;
    },
    
    setOptimalDatatableHeight: function(dt, coef) {
        //
        var self = this;
        var dtHeight = 0;
        var tableEl = dt.getBdContainerEl();
        var maxHeight = gt.config.accordPanelMaxHeight - coef;
        var currHeight = YAHOO.util.Dom.getRegion(tableEl.children[0]).height;
        
        //
        if(currHeight > maxHeight) {
            dtHeight = maxHeight;
        } else {
            if(currHeight < 100) {
                dtHeight = 100 + Giscuit.Util.getScrollBarWidth();
            } else {
                dtHeight = currHeight + Giscuit.Util.getScrollBarWidth();
            }
        }
        dt.set('height', dtHeight + 'px');
        
        //
        if (YAHOO.env.ua.ie) {
            tableEl.style.height = dt.get('height');
        }
        
        //
        dt.unsubscribe("renderEvent");
        dt.subscribe("renderEvent", function() {
            self.setOptimalDatatableHeight(dt, coef);
        });

    },
    
    addHelpButton: function(url, panel) {
        if(! panel) {
            panel = this.panel;
        }
        
        var button = document.createElement('a');
        button.title = gt.translate._('help');
        button.className = 'gtHelp';
        button.target = '_blank';
        button.href = url;
        
        YAHOO.util.Dom.insertAfter(button,
            panel.firstChild.firstChild.nextSibling
        );
    },

    CLASS_NAME: "Giscuit.Control"
});