/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editgeom.js 2573 2012-04-25 16:35:06Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        if(gt.config.layout != 'desktop') {
            return;
        }
        
        gt.control.historymanage = new Giscuit.Control.Historymanage();
        gt.control.historydata = new Giscuit.Control.Historydata();
        
        gt.control.editgeom = new Giscuit.Control.Editgeom();
    }
});

/**
 * Class: Giscuit.Control.Editgeom
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Editgeom = OpenLayers.Class(Giscuit.Control, {

    button: null,
    
    controls: null,

    controlPanel: null,
    
    secondaryControlPanel: null,
    
    editLayer: null,
    
    loaded: false,
    
    lastSelectedNode: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        if(! gt.pluginParams.editingAvailable) {
            return;
        }
        
        this.initializeTouchOffset();
        
        var onClick = function (type, args, item) {
            var label = item.cfg.getProperty('text');
            self.button.set('label', label);
            
            if(this.id == 'gtDatamanageDownloadControl') {
                if(gt.control.historydata.active) {
                    gt.control.historydata.deactivate();
                } else {
                    gt.control.historydata.downloadActivate();
                }
            } else if(this.id == 'gtDatamanageUploadControl') {
                gt.control.historydata.uploadActivate();
            } else if(this.id == 'gtDatamanageActivateControl') {
                gt.control.historydata.activateActivate();
            } else if(this.id == 'gtDatamanageHistoryControl') {
                gt.control.historymanage.activate();
            } else if(this.id == 'gtEditgeomControl') {
                if(self.active) {
                    self.deactivate();
                } else {
                    self.activate();
                }
            } 
        };
        
        var menu = [];
        menu.push({
            id: 'gtEditgeomControl' ,
            text: gt.map.getControlIcon(6, gt.translate._('editing')),
            onclick: {fn: onClick}
        });
        
        if(gt.pluginParams.historymanageIsAllowed) {
            menu.push({
                id: 'gtDatamanageHistoryControl' ,
                text: gt.map.getControlIcon(9, gt.translate._('history')),
                onclick: {fn: onClick}
            });
            
            menu.push({
                id: 'gtDatamanageDownloadControl' ,
                text: gt.map.getControlIcon(2, gt.translate._('download')),
                onclick: {fn: onClick}
            });
            
            menu.push({
                id: 'gtDatamanageUploadControl' ,
                text: gt.map.getControlIcon(21, gt.translate._('upload')),
                onclick: {fn: onClick}
            });
        }
        
        if(gt.pluginParams.historydataActivatesubmitIsAllowed) {
            menu.push({
                id: 'gtDatamanageActivateControl' ,
                text: gt.map.getControlIcon(0, gt.translate._('activate')),
                onclick: {fn: onClick}
            });
        }
        
        this.button = new YAHOO.widget.Button({
            type: 'menu', 
            label: menu[0].text,
            'menu': menu,
            title: gt.translate._('editing'), 
            container: 'gtMapControls',
            lazyloadmenu: false,
            'id': 'gtEditControl'
        });  
        
    },
    
    initializeTouchOffset: function () {
        this.touchOffsetToleranceX = 0;
        this.touchOffsetToleranceY = 0;
            
        if(gt.config.touchOffset == 2) {
            this.touchOffsetToleranceX = -35;
            this.touchOffsetToleranceY = -35;
            
        } else if(gt.config.touchOffset == 3) {
            this.touchOffsetToleranceX = -45;
            this.touchOffsetToleranceY = -45;
            
        } else if(gt.config.touchOffset == 4) {
            this.touchOffsetToleranceX = 35;
            this.touchOffsetToleranceY = -35;
            
        } else if(gt.config.touchOffset == 5) {
            this.touchOffsetToleranceX = 45;
            this.touchOffsetToleranceY = -45;
        }
    },
        
    activate: function (callback) {
        //
        if(typeof(callback) != 'function') {
            callback = function(){};
        }
        
        gt.map.deactivateAll();
        
        gt.map.globalSelectControl.deactivate();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        if (activated) {         
            this.enableEditing(callback);
        }
        
        return activated;
    },
    
    deactivate: function (callback, sendDisableRequest) {
        //
        if(typeof(sendDisableRequest) == 'undefined' || sendDisableRequest === true) {        
            sendDisableRequest = true;
        } else {
            sendDisableRequest = false;
        }
        
        //
        if(typeof(callback) != 'function') {
            callback = function(){};
        }
        
        gt.map.globalSelectControl.activate();
        
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
                
        if (deactivated) {
            if(this.controlPanel) {
                this.controlPanel.div.style.visibility = 'hidden';
                this.secondaryControlPanel.div.style.visibility = 'hidden';
            }
            
            if(this.editLayer && this.editLayer.map) {
                this.editLayer.events.un({
                    'loadend': this.loadend,
                    'loadstart': this.loadstart,
                    'beforefeaturemodified': this.editAction,
                    'afterfeaturemodified': this.editUnselect,
                    scope: this
                });
            }
            
            //Deactivate snap
            if (typeof(gt.snap.vector) != 'undefined') {
                gt.snap.vector.deactivate();
            }
            if(typeof(gt.layer.snap) != 'undefined') {
                gt.layer.snap.setVisibility(false);
            }
            
            if(sendDisableRequest) {
                this.disableEditing(callback);
            } else {
                if(this.editLayer && this.editLayer.map) {
                    this.editLayer.setVisibility(false);
                }
                setTimeout(function() {
                    callback.call();
                }, 500);
            }
            
            if (gt.control.editattr) {
                gt.control.editattr.deactivate();
                gt.control.editattr.destroy();
                gt.control.editattr = null;
            }

            for (var obj in this.controls) {
                this.controls[obj].deactivate();
                this.controls[obj].destroy();
            }
            
            if (this.controlPanel) {
                try {
                    this.controlPanel.destroy();
                    this.controlPanel = null;
                } catch(e) {
                    debug.warn(e.message);
                }
            }
            
            if (this.secondaryControlPanel) {
                try {
                    this.secondaryControlPanel.destroy();
                    this.secondaryControlPanel = null;
                } catch(e) {
                    debug.warn(e.message);
                }
            }
            
            //Uncheck button
            if(this.button) {
                YAHOO.util.Dom.removeClass(
                    this.button.get('element'), 
                    'yui-checkbox-button-checked'
                );
            }
        }
                
        return deactivated;
    },
    
    enableEditing: function(callback) {
        //Check button
        this.button.set("label", Giscuit.Util.getMenuItemById(this.button, 'gtEditgeomControl').cfg.getProperty('text'));
        YAHOO.util.Dom.addClass(
            this.button.get('element'),
            'yui-checkbox-button-checked'
        );
        
        var enableCallback = {
            success: function(o) {
                try {
                    var responseText = YAHOO.lang.JSON.parse(o.responseText);
                } catch(e) {
                    Giscuit.Util.parseFailure(o, e);
                    return;
                }
                
                if (responseText.replyCode != 200) {
                    Giscuit.Util.invalidReplyCode(responseText);
                    return;
                }
                
                callback.call();
                
                this.initializeControls(responseText);
            } ,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
       
        if (gt.control.layers.getEditLayer()) {
            this.editLayer = gt.layer.edit;
            var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/privatelayers/editenable/rand/" + Math.random(), enableCallback);
        } else {
            gt.tooltip.cfg.setProperty('text', gt.translate._('noEditLayerSelected'));
            gt.tooltip.showTooltip();
            this.deactivate(callback, false);
        }
    },
    
    disableEditing: function(callback) {
        var disableCallback = {
            success: function() {
                callback.call();
                
                if(this.editLayer) {
                    this.editLayer.setVisibility(false);
                    gt.map.refresh();
                }
            },
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/privatelayers/editdisable/rand/" + Math.random(), disableCallback);
    },
    
    addVertexToSketchCallback: function(point, geometry) {
        //
        var handler = addControl = this.controls.addControl.handler;
        if(handler.CLASS_NAME == 'OpenLayers.Handler.Point') {
            return;
        }
        
        if(! point && ! geometry) {
            point = handler.point.geometry;
            if(handler.polygon) {
                geometry = handler.polygon.geometry;
            } else if (handler.line) {
                geometry = handler.line.geometry;
            }
        }
        
        var isPolygon = geometry instanceof OpenLayers.Geometry.Polygon;
        var isLine = geometry instanceof OpenLayers.Geometry.LineString;
        
        //Finalize
        if(isPolygon && geometry.getVertices().length > 3 ||
            isLine && geometry.getVertices().length > 2) {
            
            this.controls.finalizeControl.panel_div.className = 'olControlFinalizeItemActive olButton';
        } else {
            this.controls.finalizeControl.panel_div.className = 'olControlFinalizeItemInactive olButton';
        }
        
        //
        this.controls.undovertexControl.panel_div.className = 'olControlUndovertexItemActive olButton';
    },
        
    /**
     * Add
     */
    addControl: function(type) {
        
        var modifyFeatureXY = function(handler, x, y) {
            var lonlat = new OpenLayers.LonLat(x, y);
            var pixel = gt.map.getLayerPxFromLonLat(lonlat);
            handler.modifyFeature(pixel, true);
        };
        
        var undo = function() {
            var geometry = this.line.geometry;
            var components = geometry.components;
            var index = this.getCurrentPointIndex() - 1;
            var target = components[index];
            var undone = geometry.removeComponent(target);
            if (undone) {
                if (!this.redoStack) {
                    this.redoStack = [];
                }
                this.redoStack.push(target);
                this.drawFeature();
                
                modifyFeatureXY(this, components[index - 1].x, components[index - 1].y);
            }
            return undone;
        };
        
        //Add polygon
        if (type == 'Polygon') {
            this.controls.addControl = new OpenLayers.Control.DrawFeature(this.editLayer, OpenLayers.Handler.Polygon, {
                'displayClass': 'olControlAdd',
                title: gt.translate._('addFeature'),
                callbacks: {
                    'point': Giscuit.Util.bind(this.addVertexToSketchCallback, this)
                }
            });
            
            if(gt.config.layout != 'desktop') {
                this.controls.addControl.insertXY = function(x, y) {
                    modifyFeatureXY(this.handler, x, y);
                    if (this.handler && this.handler.line) {
                        this.handler.insertXY(x, y);
                    }
                };
                this.controls.addControl.handler.undo = undo;
            }
            
        //Add Line
        } else if (type == 'Line') {
            this.controls.addControl = new OpenLayers.Control.DrawFeature(this.editLayer, OpenLayers.Handler.Path, {
                'displayClass': 'olControlAdd',
                title: gt.translate._('addFeature'),
                callbacks: {
                    'point': Giscuit.Util.bind(this.addVertexToSketchCallback, this)
                }
            });
            
            if(gt.config.layout != 'desktop') {
                this.controls.addControl.insertXY = function(x, y) {
                    modifyFeatureXY(this.handler, x, y);
                    if (this.handler && this.handler.line) {
                        this.handler.insertXY(x, y);
                    }
                };
                this.controls.addControl.handler.undo = undo;
            }
        
        //Add Point
        } else if (type == 'Point') {
            this.controls.addControl = new OpenLayers.Control.DrawFeature(this.editLayer, OpenLayers.Handler.Point, {
                'displayClass': 'olControlAdd',
                title: gt.translate._('addFeature'),
                callbacks: {
                    'point': Giscuit.Util.bind(this.addVertexToSketchCallback, this)
                }
            });
            
            this.controls.addControl.insertXY = function(x, y) {
                modifyFeatureXY(this.handler, x, y);
                this.handler.finalize();
            };
        }
        
        //Events
        this.controls.addControl.events.on({
            "featureadded": this.addAction,
            "activate": function() {
                if(gt.control.editattr) {
                    gt.control.editattr.destroy();
                    gt.control.editattr = null;
                }
                this.clearInsertedFeatures();
            },
            scope: this
        });
        
        //
        this.controlPanel.addControls(this.controls.addControl);
        
    },
    
    addAction: function (evt) {
        this.controls.finalizeControl.panel_div.className = 'olControlFinalizeItemInactive olButton';
        
        if(! this.loaded) {
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        //Check if valid Polygon
        if(evt.feature.geometry instanceof OpenLayers.Geometry.Polygon
            && evt.feature.geometry.getVertices().length < 3) {
        
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        //Check if valid Line
        if(evt.feature.geometry instanceof OpenLayers.Geometry.LineString
            && evt.feature.geometry.getVertices().length == 2
            && evt.feature.geometry.components[0].distanceTo(evt.feature.geometry.components[1]) == 0
            ) {
        
            this.editLayer.removeFeatures(evt.feature, true);
            return;
        }
        
        if(gt.control.editattr) {
            gt.control.editattr.destroy();
            gt.control.editattr = null;
        }
        
        if(gt.config.layout != 'desktop') {
            gt.control.editattr = new Giscuit.Mobile.Control.Editattr(evt.feature, 0);
        } else {
            gt.control.editattr = new Giscuit.Control.Editattr(evt.feature, 0);
        }
        
        
        this.controls.addControl.deactivate();
    },
    
    /**
     * Finalize
     */
    finalizeControl: function(type) {
        this.controls.finalizeControl = new OpenLayers.Control({
            'displayClass': 'olControlFinalize',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                if(panelDiv.className.indexOf('olControlFinalizeItemActive') === -1) {
                    return;
                }
                
                this.controls.addControl.handler.finishGeometry();
            }, this),
            title: gt.translate._('finalizeSketch')
        });
        
        //
        this.controls.addControl.events.on({
            "activate": function() {
                if(type == 'Point') {
                    return;
                }
                
                panelDiv.className = 'olControlFinalizeItemInactive olButton';
                panelDiv.style.display = 'block';
            },
            "deactivate": function() {
                panelDiv.style.display = 'none';
            },
            scope: this
        });
        
        //
        this.secondaryControlPanel.addControls(this.controls.finalizeControl);
        var panelDiv = this.controls.finalizeControl.panel_div; 
        panelDiv.style.display = 'none';
    },
    
    /**
     * Add gpsnode
     */
    addgpsnodeControl: function(type) {
        this.controls.addgpsnodeControl = new OpenLayers.Control({
            'displayClass': 'olControlAddgpsnode',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                var panelDiv = this.controls.addgpsnodeControl.panel_div;
                
                if(! this.controls.addControl.active) {
                    return;
                }
                
                //Check if geolocation is active
                if(! gt.control.geolocation.point) {
                    if(gt.config.layout != 'desktop') {
                        Giscuit.Util.jqmMessage(gt.translate._('enableGeolocationTool'));
                    } else {
                        gt.tooltip.cfg.setProperty('text', gt.translate._('enableGeolocationTool'));
                        gt.tooltip.showTooltip();
                    }
                    
                    return;
                }
                
                //Check if point precision meets maximum gps accuracy 
                var maxGpsAccuracy = gt.control.layers.getEditLayer().max_gps_accuracy;
                if (maxGpsAccuracy && gt.control.geolocation.coords.accuracy > maxGpsAccuracy &&
                    ! confirm(gt.translate._('gpsAccuracyExceedsLimit').replace("%1$s", maxGpsAccuracy))) {
                    return;
                }
                
                gt.map.deactivateSnap();
                this.controls.addControl.insertXY(
                    gt.control.geolocation.point.geometry.x,
                    gt.control.geolocation.point.geometry.y
                );
                gt.map.activateSnap();
                
                this.addVertexToSketchCallback();
                
            }, this),
            title: gt.translate._('addVertexAtCurrentPosition')
        });
        
        //
        this.controls.addControl.events.on({
            "activate": function() {
                this.controls.addgpsnodeControl.panel_div.style.display = 'block';
            },
            "deactivate": function() {
                this.controls.addgpsnodeControl.panel_div.style.display = 'none';
            },
            scope: this
        });
        
        //
        this.secondaryControlPanel.addControls(this.controls.addgpsnodeControl);
        this.controls.addgpsnodeControl.panel_div.style.display = 'none';
    },
    
    /**
     * Undovertex 
     */
    undovertexControl: function(type) {
        var addControl = this.controls.addControl;
        
        this.controls.undovertexControl = new OpenLayers.Control({
            'displayClass': 'olControlUndovertex',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                if(panelDiv.className.indexOf('olControlUndovertexItemActive') === -1) {
                    return;
                }
                
                //
                var vertextCount = addControl.handler.getCurrentPointIndex();
                addControl.undo();
                if(vertextCount == 1) {
                    panelDiv.className = 'olControlUndovertexItemInactive olButton';
                    addControl.deactivate();
                    addControl.activate();
                } else {
                    this.addVertexToSketchCallback();
                }
                
            }, this),
            title: gt.translate._('undoVertex')
        });
        
        //
        addControl.events.on({
            "activate": function() {
                if(type == 'Point') {
                    return;
                }
                
                panelDiv.className = 'olControlUndovertexItemInactive olButton';
                panelDiv.style.display = 'block';
            },
            "deactivate": function() {
                panelDiv.style.display = 'none';
            },
            scope: this
        });
        
        //
        this.secondaryControlPanel.addControls(this.controls.undovertexControl);
        var panelDiv = this.controls.undovertexControl.panel_div; 
        panelDiv.style.display = 'none';
    },
    
    /**
     * Edit
     */
    editControl: function() {
        var self = this; 
       
        var title = gt.translate._('editFeature');
        if(gt.config.layout != 'desktop') {
            title = gt.translate._('editGeometry');
        } 
        
        //
        this.controls.editControl = new OpenLayers.Control.ModifyFeature(this.editLayer, {
            'displayClass': 'olControlEdit',
            clickout: false,
            toggle: false,
            mode: OpenLayers.Control.ModifyFeature.RESHAPE | OpenLayers.Control.ModifyFeature.DRAG,
            title: title,
            vertexRenderIntent: "vertex"
        });
           
        //Events
        this.controls.editControl.events.on({
            'activate': function() {
                if(gt.control.editattr) {
                    gt.control.editattr.destroy();
                    gt.control.editattr = null;
                }
                this.clearInsertedFeatures();
            },
            scope: this
        });
        
        this.editLayer.events.on({
            'loadend': this.loadend,
            'loadstart': this.loadstart,
            'beforefeaturemodified': this.editAction,
            'afterfeaturemodified': this.editUnselect,
            scope: this
        });
        
        //Set move vertex color
        this.controls.editControl.collectDragHandle = function() {
            var geometry = this.feature.geometry;
            var center = geometry.getBounds().getCenterLonLat();
            var originGeometry = new OpenLayers.Geometry.Point(
                center.lon, center.lat
            );
                    
            //Custom code
            var origin = new OpenLayers.Feature.Vector(
                originGeometry,
                null,
                {
                    pointRadius: gt.map.pointRadius, 
                    fillColor: "yellow", 
                    fillOpacity: 0.4, 
                    strokeColor: "yellow", 
                    strokeWidth: 3
                }
            );
            
            originGeometry.move = function(x, y) {
                OpenLayers.Geometry.Point.prototype.move.call(this, x, y);
                geometry.move(x, y);
            };
            origin._sketch = true;
            origin._center = true;
            this.dragHandle = origin;
            
            this.layer.addFeatures([this.dragHandle], {silent: true});
        };

        //
        this.controls.editControl.handlers.drag.up = function(evt) {
            var feature = self.editLayer.getFeatureFromEvent(evt);
            if (feature && feature._sketch) {
                self.lastSelectedNode = feature;
                self.events.triggerEvent("vertexselected", feature);
            }
        };
        
        //
        this.controls.editControl.events.on({
            "activate": function() {
                self.lastSelectedNode = null;
            }
        });
        
        //
        this.controlPanel.addControls(this.controls.editControl);
    },
        
    editAction: function(evt) {
        //
        if(evt.feature == this.controls.editControl.feature) {
            return;
        }
        
        //
        if(evt.feature.state) {
            return;
        }
        
        //
        evt.feature.originalGeometry = evt.feature.geometry.clone();
    
        //
        if (gt.control.editattr) {
            gt.control.editattr.destroy();
            gt.control.editattr = null;
        }
        gt.control.editattr = new Giscuit.Control.Editattr(evt.feature, 0);
        
    },
    
    editUnselect: function(evt) {
        if(evt.feature.state == OpenLayers.State.UPDATE && 
            evt.feature.originalGeometry != undefined) {

            this.editLayer.removeFeatures(evt.feature, true);

            //Revert unsaved geometry
            if(evt.feature.geometry) {
                evt.feature.geometry.destroy();
                evt.feature.geometry = evt.feature.originalGeometry.clone();
            }
            evt.feature.state = null;
            
            this.editLayer.addFeatures(evt.feature, true);
        }
        
        //
        if(this.controls.editsaveControl != undefined) {
            this.controls.editsaveControl.panel_div.className = 'olControlEditsaveItemInactive olButton';
        }
    },
    
    /**
     * Edit gpsnode
     */
    editgpsnodeControl: function(type) {
        var panelDiv = null;
        
        this.controls.editgpsnodeControl = new OpenLayers.Control({
            'displayClass': 'olControlEditgpsnode',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                //Control click
                if(panelDiv.className.indexOf('olControlEditgpsnodeItemActive') === -1) {
                    return;
                }
                
                //
                if(! gt.control.geolocation.point) {
                    if(gt.config.layout != 'desktop') {
                        Giscuit.Util.jqmMessage(gt.translate._('enableGeolocationTool'));
                    } else {
                        gt.tooltip.cfg.setProperty('text', gt.translate._('enableGeolocationTool'));
                        gt.tooltip.showTooltip();
                    }
                    return;
                }
                
                //
                var maxGpsAccuracy = gt.control.layers.getEditLayer().max_gps_accuracy;
                if (maxGpsAccuracy && gt.control.geolocation.coords.accuracy > maxGpsAccuracy &&
                    ! confirm(gt.translate._('gpsAccuracyExceedsLimit').replace("%1$s", maxGpsAccuracy))) {
                    return;
                }
                
                //
                var node = this.lastSelectedNode;
                if(! node) {
                    node = this.controls.editControl.feature;
                }
                
                //
                var pixelPos = gt.map.getPixelFromLonLat(new OpenLayers.LonLat(
                    gt.control.geolocation.point.geometry.x,
                    gt.control.geolocation.point.geometry.y
                ));
                
                pixelPos.x -= this.touchOffsetToleranceX;
                pixelPos.y -= this.touchOffsetToleranceY;
                gt.map.deactivateSnap();
                this.controls.editControl.dragVertex(node, pixelPos);
                this.controls.editControl.dragComplete(node);
                gt.map.activateSnap();
                
                disable();
            }, this),
            title: gt.translate._('moveVertexToCurrentPosition')
        });
        
        //Activate panel
        this.controls.editControl.events.on({
            "activate": function() {
                disable();
                panelDiv.style.display = 'block';
            },
            "deactivate": function() {
                panelDiv.style.display = 'none';
            },
            scope: this
        });
        
        //Select vertex
        this.events.register("vertexselected", this, function(feature) {
            if(! feature._index && ! feature._center) {
                enable();
            }
        });
                
        this.editLayer.events.on({
            //Select vertex
            'beforefeaturemodified': function() {
                if(type == 'Point') {
                    enable();
                }
            },
            //Unselect feature
            'afterfeaturemodified': function() {
                disable();
            }
        });
        
        //Layer reload
        this.editLayer.events.on({'loadstart': function() {
            disable();
        }});
        
        //Vertex removed
        this.editLayer.events.on({
            'vertexremoved': function(evt) {
                if(evt.vertex.id == this.lastSelectedNode.geometry.id) {
                    disable();
                }
            },
            scope: this
        });
        
        //Add control to panel
        this.secondaryControlPanel.addControls(this.controls.editgpsnodeControl);
        panelDiv = this.controls.editgpsnodeControl.panel_div;
        panelDiv.style.display = 'none';
        
        //Disable
        var disable = function() {
            panelDiv.className = 'olControlEditgpsnodeItemInactive olButton';
        };
        
        //Enable
        var enable = function() {
            panelDiv.className = 'olControlEditgpsnodeItemActive olButton';
        };
    },
    
    /**
     * Remove vertex
     */
    removevertexControl: function(type) {
        var panelDiv = null;
                
        this.controls.removevertexControl = new OpenLayers.Control({
            'displayClass': 'olControlRemovevertex',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                //Control click
                if(panelDiv.className.indexOf('olControlRemovevertexItemActive') === -1) {
                    return;
                }
                this.removeVertex(this.controls.editControl, this.lastSelectedNode);
                disable();
                this.controls.editgpsnodeControl.panel_div.className = 'olControlEditgpsnodeItemInactive olButton';
            }, this),
            title: gt.translate._('removeVertex')
        });
                
        //Activate panel
        this.controls.editControl.events.on({
            "activate": function() {
                if(type == 'Point') {
                    return;
                }
                
                disable();
                panelDiv.style.display = 'block';
            },
            "deactivate": function() {
                panelDiv.style.display = 'none';
            },
            scope: this
        });
        
        //Select vertex
        this.events.register("vertexselected", this, function(feature) {
            if(! feature._index && ! feature._center) {
                enable();
            }
        });
        
        //Unselect feature
        this.editLayer.events.on({
            'afterfeaturemodified': function() {
                disable();
            }
        });
        
        //Layer reload
        this.editLayer.events.on({'loadstart': function() {
            disable();
        }});
        
        //Vertex removed
        this.editLayer.events.on({
            'vertexremoved': function(evt) {
                if(evt.vertex.id == this.lastSelectedNode.geometry.id) {
                    disable();
                }
            },
            scope: this
        });
        
        //Add control to panel
        this.secondaryControlPanel.addControls(this.controls.removevertexControl);
        panelDiv = this.controls.removevertexControl.panel_div;
        panelDiv.style.display = 'none';
        
        //Disable
        var disable = function() {
            panelDiv.className = 'olControlRemovevertexItemInactive olButton';
        };
        
        //Enable
        var enable = function() {
            panelDiv.className = 'olControlRemovevertexItemActive olButton';
        };
    },
    
    /**
     * Delete
     */
    deleteControl: function() {
        this.controls.deleteControl = new OpenLayers.Control.SelectFeature(this.editLayer, {
            'displayClass': 'olControlDelete',
            title: gt.translate._('deleteFeature')
        });
        this.controls.deleteControl.events.on({
            "activate": function() {
                if(gt.control.editattr) {
                    gt.control.editattr.destroy();
                    gt.control.editattr = null;
                }
                this.clearInsertedFeatures();
            },
            "featurehighlighted": this.deleteAction,
            scope: this
        });
        
        //
        this.controlPanel.addControls(this.controls.deleteControl);
    },
    
    deleteAction: function (evt) {
        var deleteCallback = {
            success: function() {
                this.events.triggerEvent("deletefeature", evt.feature.attributes.gid);
                this.backActivate();
            },
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        setTimeout(Giscuit.Util.bind(function() {
            var answer = confirm(gt.translate._('deleteConfirm'));
            if (answer) {
                this.editLayer.removeFeatures(evt.feature);
                
                var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom/delete/id/" + evt.feature.attributes.gid, deleteCallback);
            } else {
                this.controls.deleteControl.unselect(evt.feature);
            }
        }, this), 100);
    },
    
    /**
     * Back
     */
    backControl: function(history, changesCount) {
        if(! history) {
            return;
        }
        
        this.controls.backControl = new OpenLayers.Control({
            'displayClass': 'olControlBack',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                if(this.controls.backControl.panel_div.className == 'olControlBackItemActive olButton') {
                    this.backAction();
                }
            }, this),
            title: gt.translate._('undoFeature')
        });
        
        //
        this.controlPanel.addControls(this.controls.backControl);
    },
    
    backAction: function() {
        var backCallback = {
            success: this.backSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom/back", backCallback);
    } ,
    
    backActivate: function() {
        if(this.controls.backControl == undefined) {
            return false;
        }
        
        this.controls.backControl.panel_div.className = 'olControlBackItemActive olButton';
        
        return true;
    },
    
    backDeactivate: function() {
        if(this.controls.backControl == undefined) {
            return false;
        }
        
        this.controls.backControl.panel_div.className = 'olControlBackItemInactive olButton';
        
        return true;
    },
    
    backSuccess: function (o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        if(responseText.changesCount == 0) {
            this.backDeactivate();
        } else {
            this.backActivate();
        }
        
        this.editLayer.mergeNewParams( {
            'rand' : Math.random()
        });
    },
    
    /**
     * Layer name
     */
    layerNameControl: function() {
        this.controlPanel.addLayerNameControl = function() {
            if(! this.div.children.length) {
                return;
            }
            
            var layers = gt.control.layers.records;
            var layerNamePH = document.createElement('select');
            var layersData = [];
            for (var i = 0; i < layers.length; i++) {
                layersData.push({
                    id: layers[i].id,
                    name: layers[i].name,
                    order: layers[i].order,
                    edit: layers[i].edit,
                    view: layers[i].view
                });
            }
            layersData = gt.map.orderRecordsAsc(layersData);
            
            for (var i = 0; i < layersData.length; i++) {
                if (layersData[i].edit != 'none' && layersData[i].view) {
                    var option = document.createElement("option");
                    option.value = layersData[i].id; 
                    option.text = layersData[i].name;
                    
                    if (layersData[i].edit) {
                        option.selected = true;
                    }
                    
                    try {
                        layerNamePH.add(option, null);
                    } catch(ex) {
                        layerNamePH.add(option);
                    }
                }
            }

            layerNamePH.className = 'olControlLayerName';
            
            this.div.appendChild(layerNamePH);

            //Add event listener for quick edit switch
            if (gt.config.layout == 'desktop') {
                YAHOO.util.Event.addListener(layerNamePH, "change", function(oArgs) {
                    this.disabled = true;
                    var layerId = this.options[this.selectedIndex].value;
                    gt.control.layers.setSettings('edit', layerId, true);
                });
            } else {
                $(layerNamePH).on('change', this, function(evt) {
                    this.disabled = true;
                    var layerId = this.options[this.selectedIndex].value;
                    gt.control.layers.setSettings('edit', layerId, true);
                });
            }
            
            //Do not show layer name if delete control is not at the top line
            if(gt.config.layout != 'desktop') {
                if($(layerNamePH).position().top > 40) {
                    $(layerNamePH).remove();
                }
            }
            
        };
        this.controlPanel.redraw = function() {
            OpenLayers.Control.Panel.prototype.redraw.apply(this, arguments);
            this.addLayerNameControl();
        };
        this.controlPanel.draw = function() {
            OpenLayers.Control.Panel.prototype.draw.apply(this, arguments);
            this.addLayerNameControl();
            return this.div;
        };
    } ,
    
    invalidEditScale: function(editMaxscale) {
        gt.tooltip.cfg.setProperty('text', gt.translate._('layerIsEditableFromScale') + ' 1:' + editMaxscale);
        gt.tooltip.showTooltip();
    },
    
    //Initialize Controls
    initializeControls: function (data) {
        //Activate snap
        if(typeof(gt.layer.snap) != 'undefined') {
            gt.layer.snap.setVisibility(true);
        }
        if (typeof(gt.snap.edit) != 'undefined') {
            gt.snap.edit.activate();
        }
        
        //Style
        this.editLayer.styleMap.styles['default'].rules = [];
        this.editLayer.styleMap.styles['default'].addRules(gt.map.getStyleRules(data.sld, true));
        
        //        
        this.controls = {};

        //Invalid edit scale message
        if(gt.map.getScale() > data.edit_maxscale) {
            this.invalidEditScale(data.edit_maxscale);
        }
        
        //Initialize panels
        var controlPanelDiv = document.createElement('div');
        controlPanelDiv.className = 'gtEditgeomToolbar';
        OpenLayers.Util.getElement('gtMap').appendChild(controlPanelDiv);
        this.controlPanel = new OpenLayers.Control.Panel({
            'div': controlPanelDiv
        });
        
        var secondaryControlPanelDiv = document.createElement('div');
        secondaryControlPanelDiv.className = 'gtEditgeomToolbar gtEditgeomSecondaryToolbar';
        OpenLayers.Util.getElement('gtMap').appendChild(secondaryControlPanelDiv);
        this.secondaryControlPanel = new OpenLayers.Control.Panel({
            'div': secondaryControlPanelDiv
        });
        
        //Primary panel controls
        if (data.editgeom) {
            this.addControl(data.type);
            this.editControl();
            //this.touchOffset();
            this.deleteControl();
            this.backControl(data.history, data.changesCount);
        } else {
            this.editControl();
        }
        
        //
        if(gt.layer.edit) {
            this.editLayer.setVisibility(true);
            gt.map.refresh();
        }
        
        this.layerNameControl();
        
        //Add panels
        gt.map.addControl(this.controlPanel);
        
        //Secondary panel controls
        if (data.editgeom) {
            //
            this.finalizeControl(data.type);
            this.undovertexControl(data.type);
            this.addgpsnodeControl(data.type);
            
            //
            this.editgpsnodeControl(data.type);
            this.removevertexControl(data.type);
            
            //
            gt.map.addControl(this.secondaryControlPanel);
        }
        
        //Style controls
        var mapControls = YAHOO.util.Dom.getRegion('gtMapControls');
        var editControl = YAHOO.util.Dom.getRegion('gtEditControl');
        var region = YAHOO.util.Dom.getRegion(this.controlPanel.div);
        
        this.controlPanel.div.style.left = editControl.left - mapControls.left - region.width + editControl.width - 1 + 'px';
        this.secondaryControlPanel.div.style.left = editControl.left - mapControls.left + editControl.width - 2 + 'px';
                
        this.controlPanel.div.style.visibility = 'visibile';
        this.secondaryControlPanel.div.style.visibility = 'visibile';
        
        //
        if(data.changesCount > 0) {
            this.backActivate();
        }
    },
    
    loadend: function() {
        this.loaded = true;
        
        gt.control.editgeom.unselectAll();
        
        if(gt.control.editattr) {
            gt.control.editattr.destroy();
            gt.control.editattr = null;
        }
    },
    
    loadstart: function() {
        this.loaded = false;
    },
   
    clearInsertedFeatures: function() {
        var features = this.editLayer.features;
        for(var i = 0; i < features.length; i++) {
            if(features[i].state == OpenLayers.State.INSERT) {
                this.editLayer.removeFeatures(features[i], true);
            }
        }
    },
    
    removeVertex: function(control, vertex) {
        var control = this.controls.editControl;
        vertex.geometry.parent.removeComponent(vertex.geometry);
        control.layer.events.triggerEvent("vertexremoved", {
            vertex: vertex.geometry,
            feature: control.feature
        });
        control.layer.drawFeature(control.feature, control.standalone ? undefined : 'select');
        control.modified = true;
        control.resetVertices();
        control.setFeatureState();
        control.onModification(control.feature);
        control.layer.events.triggerEvent("featuremodified", {feature: control.feature});
    },
    
    unselectAll: function() {
        var editControl = this.controls.editControl;
        if(editControl.feature) {
            editControl.unselectFeature(editControl.feature);
        }
    },
    
    CLASS_NAME: "Giscuit.Control.Editgeom"
});