/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Geolocation.js 3231 2012-12-25 12:10:13Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        if(gt.config.layout != 'desktop') {
            return;
        }
        
        gt.control.geolocation = new Giscuit.Control.Geolocation();
    }
});

/**
 * Class: Giscuit.Control.Geolocation
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Geolocation = OpenLayers.Class(Giscuit.Control, {
    
    button: null,
    
    receivedPos: false,
    
    gl: null,
    
    watchProcess: null,
    
    circle: null,
    
    point: null,
    
    timestamp: null,
    
    coords: null,
    
    manually: false,
    
    pointStyle: {
        pointRadius: 5,
        fillColor: "#FF0000",
        strokeColor: "#FF9933",
        strokeWidth: 1
    },
    
    circleStyle: {
        fill: false,
        strokeColor: '#F00F00',
        strokeOpacity: 0.5
    },
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        //Geolocation
        if(typeof(navigator.geolocation) != 'undefined') {
            this.gl = navigator.geolocation;
        }
        
        //
        if(this.gl != null) {
            this.button = new YAHOO.widget.Button({ 
                label: gt.map.getControlIcon(8), 
                id: "gtControlGeolocation", 
                onclick: {fn: function() {
                    if(this.get('checked')) {
                        self.start();
                    } else {
                        self.stop();
                    }
                }},
                title: gt.translate._('myLocation'), 
                container: "gtMapControls",
                type: "checkbox"
            });
        }
        
        //GpsGate
        GpsGate.Client.getVersion(Giscuit.Util.bind(function(evt) {
            this.gl = 'gpsgate';
        }, this));
    },
        
    error: function(error) {
        var msg = '';
        switch(error.code) {
            case error.PERMISSION_DENIED: msg = 'geolocationDataNotShared';
            break;
        
            case error.POSITION_UNAVAILABLE: msg = 'geolocationPositionNotDetected';
            break;
            
            case error.TIMEOUT: msg = 'geolocationTimedOut';
            break;
            
            default: msg = 'geolocationUnknownError';
            break;
        }
        
        gt.tooltip.cfg.setProperty('text', gt.translate._(msg));
        gt.tooltip.showTooltip();
        
        this.stop();
    },
    
    start: function () {
        if(this.gl == 'gpsgate') {
            var self = this;
            this.watchProcess = setInterval(function() {
                GpsGate.Client.getGpsInfo(Giscuit.Util.bind(self.update, self));
            }, 1000);
        } else {
            this.watchProcess = this.gl.watchPosition(Giscuit.Util.bind(this.update, this), Giscuit.Util.bind(this.error, this), {
                'enableHighAccuracy': true ,
                'maximumAge': 0 ,
                'timeout': 3 * 60 * 1000
            });
        }
        
        if(this.button) {
            //Check button
            this.button.set('checked', true);
        }
    },
    
    stop: function () {
        if(this.watchProcess) {
            if(this.gl == 'gpsgate') {
                clearInterval(this.watchProcess);
            } else {
                this.gl.clearWatch(this.watchProcess);
            }
            this.watchProcess = null;
        }
        
        if(this.circle) {
            this.circle.destroy();
            this.circle = null;
        }
        
        if(this.point) {
            this.point.destroy();
            this.point = null;
        }
        
        if(this.button) {
            //Uncheck button
            this.button.set('checked', false);
        }
        
        this.receivedPos = false;
    },
    
    update: function (position) {
        if(this.manually) {
            return;
        }
        
        if(this.gl == 'gpsgate') {
            var gpsgatepPosition = position;
            
            var position = {
                coords: {
                    longitude: gpsgatepPosition.trackPoint.position.longitude,
                    latitude: gpsgatepPosition.trackPoint.position.latitude,
                    accuracy: gpsgatepPosition.trackPoint.precision.hdop
                }
            };
            
        } else {
            var gpsgatepPosition = null;
        }
        
        //
        var wgs = new OpenLayers.Projection("EPSG:4326");
        //FIMXE: Use EPSG:3857
        var sm = new OpenLayers.Projection("EPSG:900913");
                
        var currPosition = new OpenLayers.LonLat(position.coords.longitude, position.coords.latitude);
        currPosition.transform(wgs, gt.map.projection);
        
        var pointGeometry = new OpenLayers.Geometry.Point(currPosition.lon, currPosition.lat);
        
        if(! gt.map.maxExtent.containsLonLat(currPosition)) {
            this.stop();
            
            if(gt.config.layout == 'desktop') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('geolocationLocationOutsideMaxExtent'));
                gt.tooltip.showTooltip();
            } else {
                Giscuit.Util.jqmMessage(gt.translate._('geolocationLocationOutsideMaxExtent'));
            }
            
            return;
        }
        
        var circleRadius = position.coords.accuracy / 2;
            
        if(gt.config.units == 'dd') {
            var pointGeometrySm = pointGeometry.clone().transform(gt.map.projection, sm);
            var circleGeometry = new OpenLayers.Geometry.Polygon.createRegularPolygon(
                pointGeometrySm,
                circleRadius, 
                40,
                0
            );
            circleGeometry.transform(sm, gt.map.projection);
        } else if(gt.config.units == 'm') {
            var circleGeometry = new OpenLayers.Geometry.Polygon.createRegularPolygon(
                new OpenLayers.Geometry.Point(pointGeometry.x, pointGeometry.y),
                circleRadius, 
                40,
                0
            );
        } else {
            debug.warn('Only "dd" and "m" units are currently supported');
        }
        
        if(this.point && this.circle) {
            gt.layer.vector.removeFeatures([this.circle, this.point], {'silent': true});
        }
        
        //
        this.circle = new OpenLayers.Feature.Vector(circleGeometry, {'accuracy': position.coords.accuracy}, this.circleStyle);
        this.point = new OpenLayers.Feature.Vector(pointGeometry, {'accuracy': position.coords.accuracy}, this.pointStyle);
        this.timestamp = OpenLayers.Util.applyDefaults({}, position.timestamp);
        this.coords = OpenLayers.Util.applyDefaults({}, position.coords);
                
        gt.layer.vector.addFeatures([this.circle, this.point]);
        //HACK:
        if(! circleRadius) {
            gt.layer.vector.removeFeatures([this.circle], {'silent': true});
        }
        
        if(this.receivedPos == false && position.coords.accuracy) {
            this.zoom();
            
            //Minimum accuracy 100m
            if(position.coords.accuracy < 100) {
                this.receivedPos = true;
            }
        }
        
        this.updateCallback(position);
    },
    
    updateCallback: function(position) {
        
    },
    
    zoom: function() {
        gt.map.zoomToExtent(this.circle.geometry.getBounds());
    },

    CLASS_NAME: "Giscuit.Control.Geolocation"
});