/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Jobs.js 2744 2012-06-12 07:27:23Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.jobs = new Giscuit.Control.Jobs();
    }
});

/**
 * Class: Giscuit.Control.Jobs
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Jobs = OpenLayers.Class(Giscuit.Control, {
    
    addButton: null,
    
    editButton: null,
    
    deleteButton: null,
    
    detailsButton: null,
    
    showOnMapButton: null,
    
    formSubmit: null,
    
    formBack: null,
    
    detailsBack: null,
    
    dt: null,
    
    layer: null,
    
    tooltip: null,
    
    tooltipShown: false,

    drawControl: null,
    
    lastAdded: null,
    
    dateInstances: null,
    
    lastSelected: null,
    
    lastSelectedAction: null,
    
    zoomLevel: null,
    
    timeoutId: null,
    
    dtRowsPerPage: 10,
    
    dtTextLength: 22,
    
    dtHeightCoef: 150,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        if (! gt.pluginParams.jobsPluginIsAllowedDispatcher) {
            return;
        }
        
        this.zoomLevel = gt.config.resolutions.length - 3;
        
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon('jobs.png'), 
            id: "gtControlJobs", 
            onclick: {fn: function() {
                gt.control.jobs.activate();
            }},
            title: gt.translate._('jobs'), 
            container: "gtMapControls" 
        });
        
        var css = '.gtJobsLeftDiv {float: left; width: 33%;}' +
            '.gtJobsRightDiv {float:left; width: 66%;}' +
            '.gtJobsList {padding:0px; margin-top:0px; margin-left:0px;}' +
            '.gtJobsList > li {text-overflow: ellipsis; white-space: nowrap; overflow: hidden;}' +
            '.gtJobsHeading {margin:0; padding:0; padding-top:0.8em; clear:both;}';
        
        var styleSheet = new YAHOO.util.StyleSheet(css, this.CLASS_NAME);
    },
    
    activate: function() {
        //Deactivate all active controls
        gt.map.deactivateAll();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
    
        if (activated == true) {
            //Check button
            this.button.set('checked', true);
            
            if (this.panel == null) {
                this.create();
            } else {
                gt.accord.openPanel(this.getPanelIndex());
            }
            
            this.updateJobsList();
        } else {
            gt.accord.openPanel(this.getPanelIndex());
        }
        
        return activated;
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if (deactivated == true) {
            if(this.lastSelected) {
                this.lastSelected.layer.drawFeature(this.lastSelected, 'default');
            }
            
            if (this.lastSelectedAction) {
                this.lastSelectedAction.layer.removeFeatures([this.lastSelectedAction]);
            }
            
            if (this.drawControl != null && this.drawControl.active) {
                this.drawControl.deactivate();
            }
            
            if (this.timeoutId) {
                clearTimeout(this.timeoutId);
            }
        }
        
        return deactivated;
    },
    
    clear: function() {
        this.deactivate();

        gt.accord.unsubscribe('afterPanelOpen', this.updateDtHeight);
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    create: function() {
        //Create html
        var html = '<div id="gtJobsMainDiv">' +
            '<div id="gtJobsDatatable"></div>' +
            '<div id="gtJobsButtonsDiv">' +
                '<button type="button" id="gtJobsAddButton">' + gt.translate._('add') + '</button>' +
                '<button type="button" id="gtJobsDetailsButton">' + gt.translate._('details') + '</button>' +
                '<button type="button" id="gtJobsShowOnMapButton">' + gt.translate._('showOnMap') + '</button>' +
                '<button type="button" id="gtJobsEditButton">' + gt.translate._('edit') + '</button>' +
                '<button type="button" id="gtJobsDeleteButton">' + gt.translate._('delete') + '</button>' +
            '</div>' +
        '</div>' +
        '<div id="gtJobsFormPH" style="display: none;"></div>' +
        '<div id="gtJobsDetailsDiv" style="display: none;">' + 
            '<button type="button" id="gtJobsDetailsBackButton">' + gt.translate._('back') + '</button>' +
            '<div id="gtJobsDetailsContent"></div>' +
        '</div>';

        gt.accord.addPanel({
            label: gt.translate._('jobs'),
            content: html
        });
        this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
        this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.jobs.pkg.html');
        
        //Initialize buttons
        this.addButton = new YAHOO.widget.Button("gtJobsAddButton");
        
        this.editButton = new YAHOO.widget.Button("gtJobsEditButton");
        this.editButton.set('disabled', true);
        this.editButton.get("element").style.marginLeft = '1em';
        
        this.deleteButton = new YAHOO.widget.Button("gtJobsDeleteButton");
        this.deleteButton.set('disabled', true);
        
        this.detailsButton = new YAHOO.widget.Button("gtJobsDetailsButton");
        this.detailsButton.set('disabled', true);
        
        this.showOnMapButton = new YAHOO.widget.Button("gtJobsShowOnMapButton");
        this.showOnMapButton.set('disabled', true);
        
        this.detailsBack = new YAHOO.widget.Button("gtJobsDetailsBackButton");
        
        //Set events
        this.addButton.on("click", function (evt) {
            //Tooltip
            if (! this.tooltipShown && gt.config.tooltip == 't') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('drawPointHelp'));
                gt.tooltip.showTooltip();
                this.tooltipShown = true;
            }

            if (this.drawControl != null) {
                this.drawControl.destroy();
                this.drawControl = null;
            }
            
            this.drawControl = new OpenLayers.Control.DrawFeature(gt.layer.vector, OpenLayers.Handler.Point, {
                callbacks: {
                    "done": OpenLayers.Function.bind(this.add, this)
                }
            });
            
            gt.map.addControl(this.drawControl);
            this.drawControl.activate();
        }, null, this);
        
        this.editButton.on("click", function (evt) {
            YAHOO.util.Dom.get('gtJobsMainDiv').style.display = 'none';
            YAHOO.util.Dom.get('gtJobsFormPH').style.display = 'block';
            YAHOO.util.Dom.get('gtJobsFormPH').innerHTML = this.getLoadingImage(); 
            
            var row = this.dt.getSelectedRows();
            var id = this.dt.getRecord(row[0]).getData('id');

            var url = gt.config.httpToolPath + '/jobs/edit/id/' + id;
            
            var editCallback = {
                success: this.getFormSuccess,
                timeout: this.callbackTimeout,
                failure: this.callbackFailure,
                scope: this
            };

            YAHOO.util.Connect.asyncRequest('GET', url, editCallback);
        }, null, this);
        
        this.deleteButton.on("click", function (evt) {
            this.deleteButton.set('disabled', true);
            var row = this.dt.getSelectedRows();
            var id = this.dt.getRecord(row[0]).getData('id');

            var url = gt.config.httpToolPath + '/jobs/delete/id/' + id;
            
            var deleteCallback = {
                success: function(o) {
                    this.deleteButton.set('disabled', false);
                    
                    this.dt.getDataSource().sendRequest(this.dt.configs.generateRequest(this.dt.getState()), {
                        success: this.dt.onDataReturnInitializeTable,
                        scope: this.dt,
                        argument: this.dt.getState()
                    });
                    
                    if (this.lastSelected) {
                        this.lastSelected = null;
                    }
                    
                    if (this.lastSelectedAction) {
                        this.layer.removeFeatures([this.lastSelectedAction]);
                    }
                },
                timeout: this.callbackTimeout,
                failure: this.callbackFailure,
                scope: this
            };

            YAHOO.util.Connect.asyncRequest('GET', url, deleteCallback);
        }, null, this);
        
        this.detailsButton.on("click", function (evt) {         
            this.loadDetails();
        }, null, this);
        
        this.detailsBack.on("click", function (evt) {         
            YAHOO.util.Dom.get('gtJobsDetailsDiv').style.display = 'none';
            YAHOO.util.Dom.get('gtJobsMainDiv').style.display = 'block';
        }, null, this);
        
        this.showOnMapButton.on("click", function (evt) {
            var row = this.dt.getSelectedRows();
            var geometryStr = this.dt.getRecord(row[0]).getData('the_geom');
            var geometry = OpenLayers.Geometry.fromWKT(geometryStr);
            
            //
            if(this.lastSelected) {
                this.lastSelected.layer.drawFeature(this.lastSelected, 'default');
            }
            if (this.lastSelectedAction) {
                this.lastSelectedAction.layer.removeFeatures([this.lastSelectedAction]);
            }
            
            this.lastSelected = gt.layer.Jobs.getFeatureBy('geometry', geometryStr);
            this.lastSelected.layer.drawFeature(this.lastSelected, 'select');           
            
            gt.map.zoomToExtent(geometry.getBounds());
            gt.map.zoomTo(this.zoomLevel);
        }, null, this);
        
        //Initialize layer
        this.addLayer();
        
        //Initialize datatable
        this.createDatatable();
        
        gt.accord.openPanel(this.getPanelIndex());
    },
    
    createDatatable: function () {
        var self= this;
        
        var ds = new YAHOO.util.DataSource(gt.config.httpToolPath + "/jobs/get");
        ds.responseType = YAHOO.util.DataSource.TYPE_JSON;
        ds.responseSchema = {
            resultsList: "records",
            fields: ["id", "title", "priority", "status", "date", "deadline", "the_geom"],
            metaFields: {
                totalRecords: "totalRecords", 
                offset: "offset" 
            } 
        };
        
        var paginator = new YAHOO.widget.Paginator({
            alwaysVisible: true,
            rowsPerPage: this.dtRowsPerPage,
            nextPageLinkLabel: gt.translate._('nextPage'),
            previousPageLinkLabel: gt.translate._('previousPage'),
            firstPageLinkLabel: gt.translate._('firstPage'),
            lastPageLinkLabel: gt.translate._('lastPage')
        });
        
        var formatDisplaySize = function (elCell, oRecord, oColumn, oData) {
            if (oData.length > gt.control.jobs.dtTextLength) {
                elCell.innerHTML = oData.substring(0, gt.control.jobs.dtTextLength - 1) + '...';
            } else {
                elCell.innerHTML = oData;
            }
        };
        
        var formatStatus = function (elCell, oRecord, oColumn, oData) {
            elCell.innerHTML = gt.translate._(oData);
            var color = '#FFFFFF';
            if (oData == 'pending') {
                color = '#CC5050';
            } else if (oData == 'in_progress') {
                color = '#E69138';
            } else if (oData == 'completed') {
                color = '#38761D';
            } else if (oData == 'declined') {
                color = '#999999';
            }
            elCell.style.color = color;
        };
        
        var formatPriority = function (elCell, oRecord, oColumn, oData) {
            elCell.innerHTML = gt.translate._(oData);
            var color = '#FFFFFF';
            if (oData == 'high') {
                color = '#CC5050';
            } else if (oData == 'medium') {
                color = '#E69138';
            } else if (oData == 'low') {
                color = '#38761D';
            }
            elCell.style.color = color;
        };

        var formatDate = function(elCell, oRecord, oColumn, oData) {
            var parsedTime = parseFloat(oData) * 1000;
            var date = new Date(parsedTime);
            var now = new Date();
            var diff = ((now.getTime() - date.getTime()) / 1000);
            if (diff < 86400) {
                elCell.innerHTML = Giscuit.Util.prettyDate(parseFloat(oData) * 1000) + ' ' + gt.translate._('ago');
            } else {
                var yuiDate = YAHOO.util.Date.format(date, {
                    format: gt.config.yuiDateFormat
                });
                elCell.innerHTML = yuiDate;
            }
        };
        
        var formatDeadline = function(elCell, oRecord, oColumn, oData) {
            if (! oData) {
                elCell.innerHTML = '';
                return;
            }
            
            var parsedTime = parseFloat(oData) * 1000;
            var date = new Date(parsedTime);
            var now = new Date();
            if (date <= now) {
                elCell.style.color = '#CC5050';
                var yuiDate = YAHOO.util.Date.format(date, {
                    format: gt.config.yuiDateFormat
                });
                elCell.innerHTML = yuiDate;
            } else {
                var diff = ((date.getTime() - now.getTime()) / 1000);
                if (diff < 86400) {
                    elCell.innerHTML = gt.translate._('in') + ' ' + Giscuit.Util.prettyDate(parsedTime); 
                } else {
                    var yuiDate = YAHOO.util.Date.format(date, {
                        format: gt.config.yuiDateFormat
                    });
                    elCell.innerHTML = yuiDate;
                }
                
            }
        };
        
        var columnDefs = [ 
            {key: "title", resizeable: true, sortable: true, formatter: formatDisplaySize, label: gt.translate._('title')} ,
            {key: "status", resizeable: true, sortable: true, formatter: formatStatus, label: gt.translate._('status')} ,
            {key: "priority", resizeable: true, sortable: true, formatter: formatPriority, label: gt.translate._('priority')} ,
            {key: "date", resizeable: true, sortable: true, formatter: formatDate, label: gt.translate._('created')},
            {key: "deadline", resizeable: true, sortable: true, formatter: formatDeadline, label: gt.translate._('deadline')}
        ];
        
        var generateRequest = function(oState, oSelf) { 
            // Get states or use defaults 
            oState = oState || { pagination: null, sortedBy: {key: "date", dir: "yui-dt-desc"} }; 
            var sort = oState.sortedBy.key; 
            var dir = (oState.sortedBy.dir === YAHOO.widget.DataTable.CLASS_DESC) ? "desc" : "asc";
            var startIndex = (oState.pagination) ? oState.pagination.recordOffset : 0; 
            var results = (oState.pagination) ? oState.pagination.rowsPerPage : self.dtRowsPerPage; 
            
            // Build custom request 
            return  "/sort/" + sort + 
                "/dir/" + dir + 
                "/offset/" + startIndex + 
                "/limit/" + (startIndex + results);
        }; 

        this.dt = new YAHOO.widget.ScrollingDataTable("gtJobsDatatable", columnDefs, ds, {
            selectionMode: "single",
            width: '99.5%',
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "...",
            generateRequest: generateRequest, 
            initialRequest: generateRequest(), 
            dynamicData: true,
            sortedBy : {key:"date", dir:YAHOO.widget.DataTable.CLASS_DESC}, 
            paginator: paginator
        });
        
        gt.accord.on('afterPanelOpen', this.updateDtHeight, null, this);
        
        this.dt.subscribe("rowMouseoverEvent", this.dt.onEventHighlightRow);
        this.dt.subscribe("rowMouseoutEvent", this.dt.onEventUnhighlightRow);
        
        this.dt.subscribe("postRenderEvent", function () {
            this.setDtTooltips();
        }, null, this);
        
        //Select row
        this.dt.subscribe("cellClickEvent", function (oArgs) { 
            this.dt.onEventSelectRow(oArgs);

            if (this.editButton != null) {
                this.editButton.set('disabled', false);
            }
            
            if (this.deleteButton != null) {
                this.deleteButton.set('disabled', false);
            }
            
            if (this.detailsButton != null) {
                this.detailsButton.set('disabled', false);
            }
            
            if (this.showOnMapButton != null) {
                this.showOnMapButton.set('disabled', false);
            }
            
            var row = this.dt.getSelectedRows();
            var geometryStr = this.dt.getRecord(row[0]).getData('the_geom');
            var geometry = OpenLayers.Geometry.fromWKT(geometryStr);
            
            //
            if(this.lastSelected) {
                this.lastSelected.layer.drawFeature(this.lastSelected, 'default');
            }
            if (this.lastSelectedAction) {
                this.layer.removeFeatures([this.lastSelectedAction]);
            }
            
            this.lastSelected = gt.layer.Jobs.getFeatureBy('geometry', geometryStr);
            this.lastSelected.layer.drawFeature(this.lastSelected, 'select'); 

        }, null, this);
        
        //Double click row
        this.dt.subscribe("cellDblclickEvent", function (oArgs) { 
            this.loadDetails();
        }, null, this);

        this.dt.doBeforeLoadData = function(oRequest, oResponse, oPayload) {
            //Update layer features
            self.layer.add(oResponse.results);

            //Set total records number
            oPayload.totalRecords = oResponse.meta.totalRecords;
            //oPayload.pagination.recordOffset = oResponse.meta.offset;
            return oPayload; 
        }; 

    }, 
    
    add: function (geom) {
        this.lastAdded = new OpenLayers.Feature.Vector(geom);
        gt.layer.Jobs.addFeatures([this.lastAdded]);
        
        this.drawControl.deactivate();
        
        var url = gt.config.httpToolPath + '/jobs/add';
        
        var getFormCallback = {
            success: this.getFormSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        YAHOO.util.Connect.asyncRequest('GET', url, getFormCallback);

        YAHOO.util.Dom.get('gtJobsMainDiv').style.display = 'none';
        YAHOO.util.Dom.get('gtJobsFormPH').style.display = 'block';
        YAHOO.util.Dom.get('gtJobsFormPH').innerHTML = this.getLoadingImage(); 
    },
    
    addLayer: function() {
        if (! this.layer) {
            this.layer = new Giscuit.Layer.Jobs();
            gt.control.layers.addLayer(this.layer, {name: gt.translate._('jobs'), select: true}, 'overlay');
        }
    },
    
    getFormSuccess: function (o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        } 
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }

        YAHOO.util.Dom.get('gtJobsFormPH').innerHTML = responseText.form;
        
        this.formSubmit = new YAHOO.widget.Button('gtJobsSubmit');
        this.formBack = new YAHOO.widget.Button('gtJobsBack');
        
        this.formSubmit.on("click", function(evt) {
            this.formSubmit.set('disabled', true);

            var submitCallback = {
                upload: this.jobSubmitSuccess,
                timeout: this.callbackTimeout,
                failure: this.callbackFailure,
                scope: this
            };
            
            if (this.lastAdded) {
                var postData = 'geometry=' + this.lastAdded.geometry.toString();
            } else {
                var postData = '';
            }
            
            var url = YAHOO.util.Dom.get('gtJobsForm').action;
            
            YAHOO.util.Connect.setForm('gtJobsForm', true);
            YAHOO.util.Connect.asyncRequest('POST', url, submitCallback, postData);
            
        }, null, this);
        
        this.formBack.on("click", function(evt) {
            if (this.lastAdded) {
                gt.layer.Jobs.removeFeatures([this.lastAdded]);
            }
            
            YAHOO.util.Dom.get('gtJobsMainDiv').style.display = 'block';
            YAHOO.util.Dom.get('gtJobsFormPH').style.display = 'none';
        }, null, this);
        
        var calendarElement = YAHOO.util.Dom.get('gtJobsDeadline');
        this.addCalendar(calendarElement);
    },
    
    jobSubmitSuccess: function(o) {
        this.formSubmit.set('disabled', false);
        
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        //Remove previous form elements
        this.destroy();
        
        if (typeof responseText.form == 'undefined') {
            YAHOO.util.Dom.get('gtJobsMainDiv').style.display = 'block';
            YAHOO.util.Dom.get('gtJobsFormPH').style.display = 'none';
            
            this.dt.getDataSource().sendRequest(this.dt.configs.generateRequest(this.dt.getState()), {
                success: this.dt.onDataReturnInitializeTable,
                scope: this.dt,
                argument: this.dt.getState()
            });
            
            if (this.lastSelected) {
                this.lastSelected = null;
            }
            if (this.lastSelectedAction) {
                this.layer.removeFeatures([this.lastSelectedAction]);
            }
            
        } else {
            this.getFormSuccess(o);
        }
    },
    
    loadDetails: function() {
        //Clear previous distance events
        var distanceElements = YAHOO.util.Dom.getElementsByClassName('gtJobsDistance');
        YAHOO.util.Event.removeListener(distanceElements, "click");
        
        YAHOO.util.Dom.get('gtJobsMainDiv').style.display = 'none';
        YAHOO.util.Dom.get('gtJobsDetailsDiv').style.display = 'block';
        YAHOO.util.Dom.get('gtJobsDetailsContent').innerHTML = this.getLoadingImage(); 
        
        var row = this.dt.getSelectedRows();
        var id = this.dt.getRecord(row[0]).getData('id');
        
        var url = gt.config.httpToolPath + '/jobs/getdetails/id/' + id;
        
        var getDetailsCallback = {
            success: this.getDetailsSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        YAHOO.util.Connect.asyncRequest('GET', url, getDetailsCallback);
    },
    
    getDetailsSuccess: function(o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        var self = this;
        var job = responseText.job;
        var waitingTime = Giscuit.Util.prettyDate(job.date * 1000);
        var possibleActions = responseText.actions[job.workflow_id];
        var descendingActions = job.actions.reverse();
        var html = '<h3 class="gtJobsHeading">' + gt.translate._('details') + ':</h3>' +
        '<div class="gtJobsLeftDiv">' +
            '<ul class="gtJobsList">' +
                '<li>' + gt.translate._('description') + ':</li>' +
                '<li>' + gt.translate._('workflow') + ':</li>' +
                '<li>' + gt.translate._('address') + ':</li>' +
                '<li>' + gt.translate._('contact') + ':</li>' +
                '<li>' + gt.translate._('phone') + ':</li>' +
                '<li>' + gt.translate._('waiting') + ':</li>' +
            '</ul>' +
        '</div>' +
        '<div class="gtJobsRightDiv">' +
            '<ul class="gtJobsList">' +
                '<li title="' + job.desc + '">' + job.desc + '</li>' +
                '<li>' + job.name + '</li>' +
                '<li title="' + job.contact_address + '">' + job.contact_address + '</li>' +
                '<li title="' + job.contact_name + '">' + job.contact_name + '</li>' +
                '<li title="' + job.contact_phone + '">' + job.contact_phone + '</li>' +
                '<li>' + waitingTime + '</li>' +
            '</ul>' +
        '</div>';

        //Build history section
        if (descendingActions.length) {
            html += '<h3 class="gtJobsHeading">' + gt.translate._('history') + ':</h3>' +
            '<div class="gtJobsLeftDiv">' +
                '<ul class="gtJobsList">';
            
            //Calculate time and date of actions
            var lastDay = 0;
            for (var i = 0; i < descendingActions.length; i++) {
                //Get date of current action
                var entry = descendingActions[i];
                var entryTime = new Date(entry.date * 1000);
                var currentDay = entryTime.getDate();
                
                //If it's the first action or current action is on a different day from previous action
                if (! lastDay || currentDay != lastDay) {
                    lastDay = currentDay;
                    
                    //Show day in the yui date format
                    html += '<li><strong>' + YAHOO.util.Date.format(entryTime, {format: gt.config.yuiDateFormat}) + '</strong></li>';
                    
                    //Add an empty entry to array so the day won't be considered as action when building history
                    descendingActions.splice(i, 0, null);
                    i++;
                }
                
                //Calculate minutes and seconds for the action
                var currentHours = ( entryTime.getHours() < 10 ? "0" : "" ) + entryTime.getHours();
                var currentMinutes = ( entryTime.getMinutes() < 10 ? "0" : "" ) + entryTime.getMinutes();
                html += '<li>' + currentHours + ':' + currentMinutes + '</li>';
            }
            
            html += '</ul></div>' +
            '<div class="gtJobsRightDiv">' +
                '<ul class="gtJobsList">';
            
            //Show action info
            for (var i = 0; i < descendingActions.length; i++) {
                if (descendingActions[i]) {
                    var entry = descendingActions[i];
                    var distance = this.formatDistance(this.calculateDistance(job.the_geom, entry.the_geom));
                    var actionName = '';
                    
                    //Comment action
                    if (entry.action == -1) {
                        actionName = gt.translate._('comment');
                    }
                    
                    //Any other action
                    for (var j = 0; j < possibleActions.length; j++) {
                        if (possibleActions[j]['id'] == entry.action) {
                            actionName = possibleActions[j]['name'];
                        }
                    }
                    html += '<li title="' + entry.comment + '"><i>' + actionName + '</i> ' +
                    '<a href="javascript:void(0);" class="gtJobsDistance" geom="' + entry.the_geom + '">(' + distance + ')</a>';
                    if (entry.comment != '') {
                        html += ', ' + entry.comment;
                    }
                    html += '</li>';
                } else {
                    html += '<li></li>';
                }
            }
            
            html += '</ul></div>';
        }
        
        YAHOO.util.Dom.get('gtJobsDetailsContent').innerHTML = html; 
        
        //Set distance event
        var distanceElements = YAHOO.util.Dom.getElementsByClassName('gtJobsDistance');
        YAHOO.util.Event.addListener(distanceElements, "click", function(evt) {
            var pointGeom = OpenLayers.Geometry.fromWKT(this.getAttribute("geom"));
            var feature = new OpenLayers.Feature.Vector(pointGeom);
            
            if (self.lastSelected) {
                gt.layer.Jobs.drawFeature(self.lastSelected, 'default');
            }
            
            if (self.lastSelectedAction) {
                gt.layer.Jobs.removeFeatures([self.lastSelectedAction]);
            }
            
            self.lastSelectedAction = feature;
            gt.layer.Jobs.addFeatures([self.lastSelectedAction]); 
            gt.layer.Jobs.drawFeature(self.lastSelectedAction, 'select'); 
            
            gt.map.zoomToExtent(pointGeom.getBounds());
            gt.map.zoomTo(self.zoomLevel);
            
            return false;
        });
    },
    
    updateJobsList: function() {
        //Update each 30 seconds
        if (this.timeoutId) {
            clearTimeout(this.timeoutId);
        }
        
        this.timeoutId = setTimeout(Giscuit.Util.bind(function() {
            var isAdd = YAHOO.util.Dom.get('gtJobsFormPH').style.display != 'block';
            if (this.active && this.dt && isAdd) {
                this.dt.getDataSource().sendRequest(this.dt.configs.generateRequest(this.dt.getState()), {
                    success: this.dt.onDataReturnInitializeTable,
                    scope: this.dt,
                    argument: this.dt.getState()
                });
                
                if (this.lastSelected) {
                    this.lastSelected = null;
                }
            }
            
            this.updateJobsList();
        }, this), 30000);
    },
    
    updateDtHeight: function(args) {
        if(this.panel == args.panel && this.dt) {
            this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);
        }
    },
    
    setDtTooltips: function() {
        var rows = this.dt.getTbodyEl().rows;
        var metadataEls = [];
        for (var i = 0; i < rows.length; i++) {
            metadataEls.push(rows[i].children[0]);
            metadataEls.push(rows[i].children[3]);
            metadataEls.push(rows[i].children[4]);
        }
        metadataEls.push('gtJobsTooltip');
        
        if(this.tooltip != null) {
            this.tooltip.destroy();
        }

        this.tooltip = new YAHOO.widget.Tooltip("gtJobsTooltip", { 
            context: metadataEls,
            hidedelay: 500 ,
            showdelay: 500 ,
            autodismissdelay: 1000 * 60
        });

        this.tooltip.bringToTop();
        this.tooltip.doShow = Giscuit.Util.tooltipDoShow;
        
        this.tooltip.contextTriggerEvent.subscribe(function(type, args, dt) {
            var el = dt.getTdEl(args[0]);
            var data = dt.getRecord(el).getData();
            var column = dt.getColumn(el).key;

            if (column == 'title') {
                this.cfg.setProperty("text", data.title.replace(/\n/g, '<br>'));
            }
            if (column == 'date') {
                var date = new Date(parseFloat(data.date) * 1000);

                this.cfg.setProperty("text", Giscuit.Util.formatDate(date, gt.config.jsDatetimeFormat));
            }
            if (column == 'deadline') {
                if (data.deadline) {
                    var deadline = new Date(parseFloat(data.deadline) * 1000);
                    var deadlineStr = Giscuit.Util.formatDate(deadline, gt.config.jsDatetimeFormat);
                } else {
                    var deadlineStr = ' ';
                }
                
                this.cfg.setProperty("text", deadlineStr);
            }
        }, this.dt);
    },
    
    addCalendar: function(dateElement) {
        Giscuit.Util.text2cal(dateElement, this.dateInstances);
    },
    
    calculateDistance: function(targetGeom, startGeom) {
        //Same as mobile
        if (typeof(startGeom) == 'undefined') {
            startGeom = gt.control.geolocation.point.geometry;
        } else {
            startGeom = OpenLayers.Geometry.fromWKT(startGeom);
        }
        
        if (! startGeom) {
            return gt.translate._('unknown');
        }
        
        var wgs = new OpenLayers.Projection("EPSG:4326");
        
        var currentPoint = new OpenLayers.LonLat(startGeom.x, startGeom.y);
        currentPoint.transform(gt.map.projection, wgs);
        
        var targetPointGeom = OpenLayers.Geometry.fromWKT(targetGeom);
        var targetPoint = new OpenLayers.LonLat(targetPointGeom.x, targetPointGeom.y);
        targetPoint.transform(gt.map.projection, wgs);
        
        return OpenLayers.Util.distVincenty(currentPoint, targetPoint);
    },
    
    formatDistance: function(distance) {
        //Same as mobile
        var formatedDistance = '';
        if (distance < 1) {
            formatedDistance = parseInt(distance * 1000) + 'm';
        } else {
            formatedDistance = distance.toFixed(2) + 'km';
        }
        
        return formatedDistance;
    },
    
    destroy: function() {
        if (this.formSubmit != null) {
            this.formSubmit.destroy();
            this.formSubmit = null;
        }
        
        if (this.formBack != null) {
            this.formBack.destroy();
            this.formBack = null;
        }

        if(this.dateInstances != null) {
            for (var i = this.dateInstances.length - 1; i >= 0; i--) {
                if (this.dateInstances[i].cfg == null) {
                    this.dateInstances[i] = null;
                } else {
                    this.dateInstances[i].destroy();
                }
            }
            this.dateInstances = null;
        }
    },
    
    CLASS_NAME: "Giscuit.Control.Jobs"
});