/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Rastercut.js 3449 2013-06-13 08:20:20Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.rastercut = new Giscuit.Control.Rastercut();
    }
});

/**
 * Class: Giscuit.Control.Rastercut
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Rastercut = OpenLayers.Class(Giscuit.Control, {

    control: null,

    panel: null,
    
    button: null,

    geometry: null,
    
    submitButton: null ,
    statsBackButton: null ,
    statsSubmitButton: null ,
    statsClearButton: null ,

    dt: null,
    dtHeightCoef: 120,
    
    ds: null,

    inTxt: null,
    outTxt: null,
    
    selected : null,
    
    tooltipShown: null,

    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        if(gt.pluginParams.rastercutManageIsAllowed) {
            var onClick = function (type, args, item) {
                var label = item.cfg.getProperty('text');
                self.button.set('label', label);
                
                if(this.id == 'gtControlRastercut') {
                    if(self.active) {
                        self.deactivate();
                    } else {
                        self.activate();
                    }
                    
                } else if(this.id == 'gtControlRastercutManage') {
                    self.createStats();
                }
            };
            
            var menu = [{
                id: 'gtControlRastercut' ,
                    text: gt.map.getControlIcon(16, gt.translate._('downloadRaster')),
                    onclick: {fn: onClick}
                }, {
                    id: 'gtControlRastercutManage' ,
                    text: gt.map.getControlIcon(20, gt.translate._('downloadRasterStatistics')),
                    onclick: {fn: onClick}
                }
            ];  
                        
            this.button = new YAHOO.widget.Button({
                type: "menu", 
                label: menu[0].text,
                title: gt.translate._('downloadRaster'),
                'menu': menu, 
                container: "gtMapControls",
                lazyloadmenu: false
            });
            
        } else if (gt.pluginParams.rastercutIndexIsAllowed) {
            this.button = new YAHOO.widget.Button({ 
                label: gt.map.getControlIcon(16), 
                id: "gtControlRastercut", 
                onclick: {fn: function() {
                    gt.control.rastercut.activate();
                }},
                title: gt.translate._('downloadRaster'), 
                container: "gtMapControls",
                type: 'checkbox'
            });
        }
    },
    
    activate: function () {
        gt.map.deactivateAll();

        var activated = Giscuit.Control.prototype.activate.call(this);

        if (activated == true) {
            
            if(gt.pluginParams.rastercutManageIsAllowed) {
                //Check button
                this.button.set("label", Giscuit.Util.getMenuItemById(this.button, 'gtControlRastercut').cfg.getProperty('text'));
                YAHOO.util.Dom.addClass(
                    this.button.get('element'), 
                    'yui-checkbox-button-checked'
                );
            }
            
            //Tooltip
            if (! this.tooltipShown && gt.config.tooltip == 't') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('drawGeometryHelp'));
                gt.tooltip.showTooltip();
                this.tooltipShown = true;
            }
            
            this.control = new OpenLayers.Control.DrawFeature(
            gt.layer.vector, OpenLayers.Handler.Polygon, {
                callbacks: {
                    "done": OpenLayers.Function.bind(this.done, this)
                }
            });

            gt.map.addControl(this.control);
            this.control.activate();
        }
        
        return activated;

    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);

        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        if (deactivated == true) {
            //
            if(this.control != null) {
                this.control.destroy();
                this.control = null;
            }
            
            //
            if(gt.pluginParams.rastercutManageIsAllowed) {
                //Uncheck button
                YAHOO.util.Dom.removeClass(
                    this.button.get('element'), 
                    'yui-checkbox-button-checked'
                );
            }
            
        }
        
        return deactivated;
    },
    
    clear: function() {
        this.deactivate();
        
        gt.accord.unsubscribe('afterPanelOpen', this.updateDtHeight);
        
        var elements = ['dt' ,
            'submitButton' ,    
            'statsBackButton' ,
            'statsSubmitButton' ,
            'statsClearButton' ,
         ];
         for(var i = 0; i < elements.length; i++) {
             if(this[elements[i]] != null) {
                 this[elements[i]].destroy();
                 this[elements[i]] = null;
             }
         }
         
         //
         if(this.dateInstances) {
             for (var i = this.dateInstances.length - 1; i >= 0; i--) {
                 try {
                     this.dateInstances[i].destroy();
                 } catch(evt) {
                 }
                 this.dateInstances[i] = null;
             }
             this.dateInstances = null;
         }

         Giscuit.Control.prototype.clear.call(this);
    },

    initInput: function () {

        this.submitButton = new YAHOO.widget.Button('gtRastercutSubmit');
        this.submitButton.on("click", this.submit, null, this);

        var found = false;
        var scaleObj = YAHOO.util.Dom.get('gtRastercutScale');
        var scale = Math.round(gt.map.getScale());
        var scaleArr = [];

        //Set the scale
        for (var i = 0; i < scaleObj.options.length; i++) {
            if (scaleObj.options[i].value == scale) {
                scaleObj.selectedIndex = i;
            }
        }

    },

    submitSuccess: function (o) {

        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        if (responseText.valid == false) {
            if(responseText.form != undefined) {
                gt.accord.setBody(this.getPanelIndex(), responseText.form);
                this.initInput();
            } else {
                gt.accord.setBody(this.getPanelIndex(), responseText.message);
            }
        } else {
            var url = gt.config.httpToolPath + '/rastercut/download/file/' + responseText.file;
            
            if (YAHOO.env.ua.ie) {
                var html = gt.translate._('fileCreatedIe').replace('%1$s', url);
                gt.accord.setBody(this.getPanelIndex(), html);
            } else {
                var html = '<p>' + gt.translate._('fileCreated') + '</p>';
                gt.accord.setBody(this.getPanelIndex(), html);
                window.location = url;
            }
            
            this.deactivate();
        }

    },

    submit: function (options) {

        YAHOO.util.Dom.get('gtRastercutGeometry').value = this.geometry;

        var submitCallback = {
            success: this.submitSuccess,
            failure: this.callbackFailure,
            scope: this
        };
        
        YAHOO.util.Connect.setForm('gtRastercutForm');
        var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/rastercut", submitCallback);

        gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());

    },

    getFormSuccess: function (o) {
        gt.accord.setBody(this.getPanelIndex(), o.responseText);
        this.initInput();
    },

    done: function (geometry) {
        this.deactivate();
        this.clear();
        
        this.geometry = geometry;
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('downloadRaster') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.download-raster.pkg.html');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        gt.accord.openPanel(this.getPanelIndex());

        var getFormCallback = {
            success: this.getFormSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/rastercut/getform", getFormCallback);
    },

    /**
     * Statistics
     */
    getStatsPanelSuccess: function (o) {

        gt.accord.setBody(this.getPanelIndex(), o.responseText);
        
        this.inTxt = YAHOO.util.Dom.get("gtRastercutDateBeginId");
        Giscuit.Util.text2cal(this.inTxt, this.dateInstances);
        
        this.outTxt = YAHOO.util.Dom.get("gtRastercutDateEndId");
        Giscuit.Util.text2cal(this.outTxt, this.dateInstances);

        Giscuit.Control.prototype.deactivate.call(this);

        this.statsSubmitButton = new YAHOO.widget.Button('gtRastercutManageSubmit');
        this.statsSubmitButton.on("click", this.statsSubmit, null, this);

        this.statsClearButton = new YAHOO.widget.Button('gtRastercutManageClear');
        this.statsClearButton.on("click", this.statsClear, null, this);
        
        this.statsBackButton = new YAHOO.widget.Button('gtRastercutManageBack');
        this.statsBackButton.on("click", this.statsBack, null, this);
        YAHOO.util.Dom.get('gtRastercutManageBack').style.display = 'none';
        
    },
    
    statsBack: function() {
        YAHOO.util.Dom.get('gtRastercutManageFormPH').style.display = 'block';
        YAHOO.util.Dom.get('gtRastercutManageBack').style.display = 'none';
        YAHOO.util.Dom.get('gtRastercutManageDataTable').style.display = 'none';
    },

    createDatatable: function () {
        
        if(this.dt != null) {
            this.ds.sendRequest("?dateBegin=" + encodeURIComponent(this.inTxt.value) + 
                    "&dateEnd=" + encodeURIComponent(this.outTxt.value), {
                success : this.dt.onDataReturnInitializeTable,
                failure : this.dt.onDataReturnInitializeTable,
                scope   : this.dt,
                argument: this.dt.getState()
            });
            
            return;
        }
        
        //Define columns
        var columnDefs = [
        {
            key: "user_name",
            resizeable: true,
            sortable: true,
            label: gt.translate._('user_name')
        },
        {
            key: "area",
            resizeable: true,
            sortable: true,
            label: gt.translate._('area')
        },
        {
            key: "scale",
            resizeable: true,
            sortable: true,
            label: gt.translate._('scale')
        },
        {
            key: "date",
            resizeable: true,
            sortable: true,
            label: gt.translate._('date')
        }];

        //Create data source
        this.ds = new YAHOO.util.DataSource(gt.config.httpToolPath + "/rastercut/managedatatable");
               
        this.ds.responseType = YAHOO.util.DataSource.TYPE_JSON;
        this.ds.responseSchema = {
            resultsList: "records",
            fields: ["gid", "user_name", "area", "scale", "date", "the_geom"]
        };

        this.dt = new YAHOO.widget.DataTable("gtRastercutManageDataTable", columnDefs, this.ds, {
            initialRequest: "?dateBegin=" + encodeURIComponent(this.inTxt.value) + 
                "&dateEnd=" + encodeURIComponent(this.outTxt.value),
            scrollable: true,
            width: '99.5%',
            selectionMode: "single",
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });
        
        this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);

        this.dt.subscribe("rowMouseoverEvent", this.dt.onEventHighlightRow);
        this.dt.subscribe("rowMouseoutEvent", this.dt.onEventUnhighlightRow);

        //Select row
        this.dt.subscribe("cellClickEvent", function (oArgs) {
            this.dt.onEventSelectRow(oArgs);
            
            var row = this.dt.getSelectedRows();
            var geometry = OpenLayers.Geometry.fromWKT(this.dt.getRecord(row[0]).getData('the_geom'));
            
            if(this.selected != null) {
                gt.layer.vector.destroyFeatures([this.selected]);
                this.selected = null;
            }
            
            this.selected = new OpenLayers.Feature.Vector(geometry);
            gt.layer.vector.addFeatures([this.selected]);
            
            gt.map.zoomToExtent(geometry.getBounds());
        }, null, this);
        
        this.dt.subscribe("dataReturnEvent", function() {
            gt.map.refresh();
        }, null, this);
    },

    createStats: function () {
        this.clear();
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('downloadRaster') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.download-raster.pkg.html');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        gt.accord.openPanel(this.getPanelIndex());
        
        gt.accord.on('afterPanelOpen', this.updateDtHeight, null, this);

        var getStatsCallback = {
            success: this.getStatsPanelSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/rastercut/manage", getStatsCallback);

    },

    statsSubmit: function () {
        YAHOO.util.Dom.get('gtRastercutManageFormPH').style.display = 'none';
        YAHOO.util.Dom.get('gtRastercutManageBack').style.display = 'inline-block';
        YAHOO.util.Dom.get('gtRastercutManageDataTable').style.display = 'block';
        
        this.createDatatable();
    },

    statsClear: function () {
        var inDate = YAHOO.widget.DateMath.subtract(new Date(), YAHOO.widget.DateMath.DAY, 1);
        this.inTxt.value = YAHOO.util.Date.format(inDate, {
            format: gt.config.yuiDateFormat
        });

        this.outTxt.value = YAHOO.util.Date.format(new Date(), {
            format: gt.config.yuiDateFormat
        });
        
        var statsClearCallback = {
            success: function() {
            
            },
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var request = YAHOO.util.Connect.asyncRequest('GET', 
            gt.config.httpToolPath + "/rastercut/managedatatable" +
                "?dateBegin=" + encodeURIComponent(this.inTxt.value) + 
                "&dateEnd=" + encodeURIComponent(this.outTxt.value),
            statsClearCallback
        );
        
        gt.map.refresh();
    },
    
    updateDtHeight: function(args) {
        if(this.panel == args.panel && this.dt) {
            this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);
        }
    },

    CLASS_NAME: "Giscuit.Control.Rastercut"

});