Giscuit.Layer.Incidents = OpenLayers.Class(OpenLayers.Layer.Vector, {
    
    defaultField: 'title',
    
    searchScale: 5000,

    initialize: function(name, options) {
        //Required
        this.visibility = false;
        
        var newArguments = [];
        newArguments.push(name, options);

        //Name
        newArguments[0] = 'Incidents';
        
        //Options
        newArguments[1] = {
            //Set layer projection
            'projection': 'EPSG:4326'
        };
        
        //Call constructor
        OpenLayers.Layer.Vector.prototype.initialize.apply(this, newArguments);
        
        //Events
        this.events.on({
            'visibilitychanged': this.load
        });
    },
    
    load: function(evt, type, category) {
        if(! this.visibility) {
            return;
        }
        
        if (typeof (type) == 'undefined') {
            type = 1;
        }
        
        if (typeof (category) == 'undefined') {
            category = 0;
        }
        
        var url = gt.config.httpToolPath + '/incidents/get/type/' + type + '/category/' + category;
        OpenLayers.Request.GET({
            url: url,
            success: this.add,
            scope: this
        });
    },
    
    add: function(o) {
        try {
            if(typeof(YAHOO) != 'undefined') {
                var responseText = YAHOO.lang.JSON.parse(o.responseText);
            } else {
                var responseText = $.parseJSON(o.responseText);
            }
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
        }
        
        var defaultStyle = OpenLayers.Util.applyDefaults({
            fillOpacity: '${__getFillOpacity}' ,
            pointRadius: 8 ,
            strokeWidth: 2 ,
            fillColor: '${__getFillColor}' ,
            strokeColor: '${__getStrokeColor}'
        }, OpenLayers.Feature.Vector.style['default']);
        
        var selectStyle = OpenLayers.Util.applyDefaults({
            fillOpacity: '${__getFillOpacity}' ,
            pointRadius: 8 ,
            strokeWidth: 2 ,
            fillColor: '${__getFillColor}' ,
            strokeColor: '${__getStrokeColor}'
        }, OpenLayers.Feature.Vector.style['select']);

        
        var style = new OpenLayers.StyleMap( {
            'default': defaultStyle,
            'select': selectStyle
        });
                
        style.styles['select'].context = style.styles['default'].context = {
            '__getFillColor': function(feature) {
                var catColor = feature.attributes.category_color;
                if(! catColor || feature.renderIntent == 'select') {
                    catColor = '#66CCCC';
                }
                
                if(feature.geometry.CLASS_NAME == 'OpenLayers.Geometry.Polygon' ||
                       feature.geometry.CLASS_NAME == 'OpenLayers.Geometry.Point') {
                
                    return catColor;
                }
                
                return null;
            },
            '__getStrokeColor': function(feature) {
                var catColor = feature.attributes.category_color;
                if(! catColor || feature.renderIntent == 'select') {
                    catColor = '#66CCCC';
                }
                
                if(feature.geometry.CLASS_NAME == 'OpenLayers.Geometry.Polygon' ||
                       feature.geometry.CLASS_NAME == 'OpenLayers.Geometry.Point') {
                
                    return '#FFFFFF';
                } else {
                    return catColor;
                }
            },
            '__getFillOpacity': function(feature) {
                if(! feature.attributes.category_color) {
                    return 0.2;
                }
                
                return 1;
            }
        };
        
        this.styleMap = style;
                                
        var features = responseText.records;
        var newFeatures = [];
        
        for(var i = 0; i < features.length; i++) {
            //Geometry
            var point = new OpenLayers.Geometry.fromWKT(features[i].the_geom);
            
            var newFeature = new OpenLayers.Feature.Vector(point);
            
            //Attributes
            newFeature.attributes = features[i];
            
            if (gt.config.layout == 'desktop') {
                if (typeof newFeature.attributes.image != 'undefined') {
                    delete(newFeature.attributes.image_url);
                }
            } else {
                newFeature.attributes.category = features[i].category_name;

                if (typeof newFeature.attributes.image != 'undefined') {
                    newFeature.attributes.image = '<a target="_blank" href="' + newFeature.attributes.image_url + '">' + 
                        gt.translate._('image') + ': ' + newFeature.attributes.image_name + '</a>';
                    
                    delete(newFeature.attributes.image_url);
                }

                delete(newFeature.attributes.category_name);
            }
            
            delete(newFeature.attributes.the_geom);
            delete(newFeature.attributes.image_name);
            delete(newFeature.attributes.image_id);
            
            newFeatures.push(newFeature);
        }
        
        //Add or update elements
        this.removeAllFeatures();
        this.addFeatures(newFeatures);
                
        if (gt.config.layout == 'desktop') {
            if (typeof(gt.control.incidents) != 'undefined' && gt.control.incidents.dt) {
                gt.control.incidents.getFeaturesFromLayer();
    
                var incidentsDt = gt.control.incidents.dt;
                incidentsDt.getDataSource().sendRequest(gt.control.incidents.features, {
                    success: incidentsDt.onDataReturnInitializeTable,
                    scope: incidentsDt,
                    argument: incidentsDt.getState()
                });
                
                gt.control.incidents.setDtTooltips();
                
                if (gt.pluginParams.incidentsSetstatusIsAllowed) {
                    incidentsDt.showColumn('fname');
                    incidentsDt.showColumn('lname');
                    incidentsDt.showColumn('email');
                }
                
                gt.control.incidents.setOptimalDatatableHeight(
                    gt.control.incidents.dt, 
                    gt.control.incidents.dtHeightCoef
                );
            }
        } else {
            if($('#gtIncidents').length) {
                gt.control.incidents.buildIncidents();
            }
        }
    },

    CLASS_NAME: "Giscuit.Layer.Incidents"
});
