/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Measure.js 1230 2011-04-14 11:14:59Z ue $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.measure = new Giscuit.Mobile.Control.Measure( {
            type : 'distance'
        });
        
        gt.control.area = new Giscuit.Mobile.Control.Measure( {
            type : 'area'
        });
    }
});

/**
 * Class: Giscuit.Mobile.Control.Measure
 * 
 *  Inherits from:
 *   - <Giscuit.Control.Measure>
 */
Giscuit.Mobile.Control.Measure = OpenLayers.Class(Giscuit.Control.Measure, {
    
    controlPanel: null,

    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        if(! $('#gtMeasureText').length) {
            $('#gtMapDiv').append('<input id="gtMeasureText" type="text" />');
        }
        
        if(! $("#gtMeasureDistanceButton").length) {
            
            $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtMeasureDistanceButton">'
                + '<div class="gtMapControlsElImage" style="background-position: -256px 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('distance') + '</div>'
                + '</div></div>');
            
            $("#gtMeasureDistanceButton").on(gt.config.downClickEvt, this, function(evt) {
                //
                evt.stopImmediatePropagation();
                
                var self = gt.control.measure;
                
                if(self.active && $("#gtMeasureDistanceButton").hasClass('ui-btn-active')) {
                    $("#gtMeasureDistanceButton").removeClass('ui-btn-active');
                    self.deactivate();
                } else {
                    //
                    self.deactivate();
                    $("#gtMeasureDistanceButton").addClass('ui-btn-active');
                    
                    //Hide map controls
                    gt.init.layout.hideMapControls();
                    
                    self.type = 'distance';
                    self.activate();
                }
                
                return false;
            });
        }
            
        if(! $("#gtMeasureAreaButton").length) {
            $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtMeasureAreaButton">'
                + '<div class="gtMapControlsElImage" style="background-position: -448px 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('area') + '</div>'
                + '</div></div>');
               
            $("#gtMeasureAreaButton").on(gt.config.downClickEvt, this, function(evt) {
                //
                evt.stopImmediatePropagation();
                
                var self = gt.control.area;
                   
                if(self.active && $("#gtMeasureAreaButton").hasClass('ui-btn-active')) {
                    $("#gtMeasureAreaButton").removeClass('ui-btn-active');
                    self.deactivate();
                } else {
                    //
                    self.deactivate();
                    $("#gtMeasureAreaButton").addClass('ui-btn-active');
                    
                    //Hide map controls
                    gt.init.layout.hideMapControls();
                    
                    self.type = 'area';
                    self.activate();
                }
                
                return false;
            });
        }
        
    },
    
    activate: function () {
        var deactivateEditgeom = false;
        if(typeof(gt.control.editgeom) != 'undefined' && gt.control.editgeom.active) {
            deactivateEditgeom = gt.control.editgeom.active;
        }
        
        gt.map.deactivateAll();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
        $("#gtMeasureButton").addClass('ui-btn-active');
        
        if (activated == true) {

            if(this.type == 'distance') {
                $("#gtMeasureDistanceButton").addClass('ui-btn-active');
                var handler = OpenLayers.Handler.Path;
            } else if(this.type == 'area') {
                $("#gtMeasureAreaButton").addClass('ui-btn-active');
                var handler = OpenLayers.Handler.Polygon;
            }
            
            this.control = new OpenLayers.Control.DrawFeature(gt.layer.vector, handler, {
                callbacks: {
                    "point": OpenLayers.Function.bind(this.populate, this),
                    "done": this.done
                },
                handlerOptions: {
                    persist: true
                }
            });
            gt.map.addControl(this.control);
            this.control.activate();
            
            //Activate snap
            var activateSnap = function() {
                if(typeof(gt.layer.snap) != 'undefined') {
                    gt.layer.snap.setVisibility(true);
                }
                if (typeof(gt.snap.vector) != 'undefined') {
                    gt.snap.vector.activate();
                }
            };
            
            if(deactivateEditgeom) {
                var activateSnapTimeout = 1000;
            } else {
                var activateSnapTimeout = 100;
            }
            setTimeout(function() {
                activateSnap();
            }, 1000);
            
        }

        return activated;
    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if (deactivated == true) {
            $('#gtMeasureDistanceButton').removeClass('ui-btn-active');
            $('#gtMeasureAreaButton').removeClass('ui-btn-active');
            
            if(this.control) {
                this.control.destroy();
                this.control = null;
            }
            
            //Deactivate snap
            if (typeof(gt.snap.vector) != 'undefined') {
                gt.snap.vector.deactivate();
            }
            if(typeof(gt.layer.snap) != 'undefined') {
                gt.layer.snap.setVisibility(false);
            }
            
            $('#gtMeasureText').css('visibility', 'hidden');
        }
        
        return deactivated;
    },

    populate: function (current, geometry) {
        
        if(this.type == 'distance') {
            var result = this.getBestLength(geometry.clone());
            var text = result[0].toFixed(3) + ' ' + result[1];                                                                  ;
        } else if(this.type == 'area') {
            var result = this.getBestArea(geometry);
            var text = result[0].toFixed(3) + ' ' + result[1] + '2';
        }

        $('#gtMeasureText').css('visibility', 'visible');
        $('#gtMeasureText').val(text);
    },

    CLASS_NAME: "Giscuit.Mobile.Control.Measure"
});