<?php

class AuthControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{

    protected $_validLogin = array(
        'account' => 'test@giscuit.com' , 
        'password' => '123456'
    );

    protected $_invalidLogin = array(
        'account' => 'fail' , 
        'password' => 'fail'
    );

    protected $_signupArray = array(
        'account' => 'signup@test.com' , 
        'password' => 'password' , 
        'confirm_password' => 'password' , 
        'fname' => 'test' , 
        'lname' => 'test' ,
        'timezone' => 'Europe/Chisinau' ,
        'agreement' => 't'
    );

    public function testLoginIndexAction ()
    {
        $this->dispatch('/' . $this->_language . '/default/auth');
        $this->assertModule('default');
        $this->assertController('auth');
        $this->assertAction('index');
        $this->assertQuery('form#gtAuthLoginForm');
    }
    
    public function testDeleteExistingTestSignupUser()
    {
        $table = new Giscuit_Db_Table_Users();
        $config = Zend_Registry::get('config');
        
        $this->_testLoginActionSuccess();
        $select = $table->select();
        $select->from($config->database->schema . '.users', 'id');
        $select->where('account = ?', $this->_signupArray['account']);
        $result = $table->fetchAll($select)->current();
        
        if ( $result != null ) {
            $this->dispatch('/' . $this->_language . '/admin/users/delete/ids/' . $result->id);
            $response = $this->getResponse()->getBody();
            $response = Zend_Json::decode($response);
            $this->assertEquals($response['replyCode'], 200);
        }
    }

    public function testLoginActionSuccess ()
    {
        $this->request->setMethod('POST')->setPost($this->_validLogin);
        $this->dispatch('/' . $this->_language . '/default/auth/index');
        $this->assertObjectHasAttribute('id', Zend_Auth::getInstance()->getIdentity());
    }

    public function testLogoutAction ()
    {
        $this->testLoginActionSuccess();
        $this->dispatch('/' . $this->_language . '/default/auth/logout');
        
        $this->assertRedirectTo('/');
    }

    public function testLoginActionFailure ()
    {
        $this->request->setMethod('POST')->setPost($this->_invalidLogin);
        $this->dispatch('/' . $this->_language . '/default/auth/index');
        $this->assertFalse(Zend_Auth::getInstance()->hasIdentity());
    }

    public function testSignupIndexAction ()
    {
        $this->dispatch('/' . $this->_language . '/default/auth/signup');
        $this->assertModule('default');
        $this->assertController('auth');
        $this->assertAction('signup');
        $this->assertQuery('form#gtAuthSignupForm');
    }

    public function testSignupSubmitSuccessAction ()
    {
        $this->request->setMethod('POST')->setPost($this->_signupArray);
        $this->dispatch('/' . $this->_language . '/default/auth/signup');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $translate = Zend_Registry::get('Zend_Translate');
        $this->assertContains($translate->_('signupSuccessfull'), $response['result']);
    }

    public function testSignupSubmitUserExistsAction ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $this->request->setMethod('POST')->setPost($this->_signupArray);
        $this->dispatch('/' . $this->_language . '/default/auth/signup');
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        
        $this->assertContains($translate->_('This ID is not available'), $response['form']);
    }

}