<?php

class UsersControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{

    protected $_account = 'signup@test.com';

    protected $_editedFname = 'friendlyyy';

    protected $_wrongCol = 'unexistable';

    public function testIndexAction ()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/admin/users');
        
        $this->assertModule('admin');
        $this->assertController('users');
        $this->assertAction('index');
    }

    public function testGetAction ()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/admin/users/get');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        $this->assertNotNull($response['records']);
    
    }

    public function testUpdateActionSuccess ()
    {
        $table = new Giscuit_Db_Table_Users();
        $config = Zend_Registry::get('config');
        
        $this->_testLoginActionSuccess();
        $select = $table->select();
        $select->from($config->database->schema . '.users', 'id');
        $select->where('account = ?', $this->_account);
        $result = $table->fetchAll($select)->current();
        
        $this->dispatch('/' . $this->_language . '/admin/users/update/data/' . $this->_editedFname . '/id/' . $result->id . '/column/fname');
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
    }

    public function testAfterUpdateAction ()
    {
        $table = new Giscuit_Db_Table_Users();
        $config = Zend_Registry::get('config');
        
        $select = $table->select();
        $select->from($config->database->schema . '.users', 'fname');
        $select->where('fname = ?', $this->_editedFname);
        $result = $table->fetchAll($select)->current();
        
        $this->assertNotNull($result);
    }

    public function testUpdateActionFailure ()
    {
        $table = new Giscuit_Db_Table_Users();
        $config = Zend_Registry::get('config');
        
        $this->_testLoginActionSuccess();
        $select = $table->select();
        $select->from($config->database->schema . '.users', 'id');
        $select->where('account = ?', $this->_account);
        $result = $table->fetchAll($select)->current();
        
        $this->dispatch('/' . $this->_language . '/admin/users/update/data/' . $this->_editedFname . '/id/' . $result->id . '/column/' . $this->_wrongCol);
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 500);
    }

    public function testDeleteUserAction ()
    {
        $table = new Giscuit_Db_Table_Users();
        $config = Zend_Registry::get('config');
        
        $this->_testLoginActionSuccess();
        $select = $table->select();
        $select->from($config->database->schema . '.users', 'id');
        $select->where('account = ?', $this->_account);
        $result = $table->fetchAll($select)->current();
        
        $this->dispatch('/' . $this->_language . '/admin/users/delete/ids/' . $result->id);
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
    }

    public function testFetchAfterDeleteAction ()
    {
        $table = new Giscuit_Db_Table_Users();
        $config = Zend_Registry::get('config');
        
        $select = $table->select();
        $select->from($config->database->schema . '.users', 'account');
        $select->where('account = ?', $this->_account);
        $result = $table->fetchAll($select)->current();
        $this->assertNull($result);
    }

}